# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""RpcServer module.

Common interface for different RPC implementations.
"""


import SocketServer, BaseHTTPServer
import sys


# Select your RPC here!
# Possible values are 'XML-RPC' and 'SOAP'
rpc = 'XML-RPC'

if rpc == 'SOAP' :
    import soaplib
    # Server is the class which represents a connection to the RPC server.
    Server = soaplib.ServerProxy
    def _dumps(self, method, response) :
        return soaplib.dumps(soaplib.Response(method, None, response), 1)
    def _loads(self, data) :
        method, params = soaplib.loads(data)
        return method, params
    rpclib = soaplib

elif rpc == 'XML-RPC' :
    import xmlrpclib
    def _dumps(self, method, response) :
        return xmlrpclib.dumps(response, methodresponse=1)
    def _loads(self, data) :
        params, method = xmlrpclib.loads(data)
        return method, params
    rpclib = xmlrpclib

    class Server(xmlrpclib.Server):
        def __init__(self, uri, transport=None,
                     encoding=None, verbose=0):
            xmlrpclib.Server.__init__(self, uri, transport=None,
                                      encoding=None, verbose=0)

        def _Server__request(self, methodname, params):
            # Ugly hack, but what the heck...
            request = xmlrpclib.dumps(tuple(map(wrap_argument,params)),
                                      methodname,
                                      encoding=self.__encoding)

            response = self.__transport.request(
                self.__host,
                self.__handler,
                request,
                verbose=self.__verbose
                )

            response= tuple(map(unwrap_argument,response))

            if len(response) == 1:
                response = response[0]

            return response




Server.__nonzero__ = lambda x:1

from types import *
try:
    SUSPECT_TYPES = [StringType,UnicodeType]
    SUSPECT_TYPES.append(UnicodeType)
except:
    SUSPECT_TYPES = [StringType,]

import re
ASCII_RE = re.compile('[^\\s\x20-\x7E]')

def wrap_argument(arg):
    if type(arg) in SUSPECT_TYPES and ASCII_RE.search(arg):
        return rpclib.Binary(arg)
    return arg

def unwrap_argument(arg):
    if isinstance(arg,rpclib.Binary):
        return arg.data
    return arg


class RequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    """Abstract class for request handling on RPC servers."""

    dumps = _dumps
    loads = _loads
    rpc = rpc

    def log_message(self,format,*args):
        pass

    def do_POST(self):
        try:
            # get arguments
            data = self.rfile.read(int(self.headers["content-length"]))
            method, params = self.loads(data)
            # generate response
            try:
                response = self.call(method,
                                     tuple(map(unwrap_argument,params)))
                if type(response) != type(()):
                    
                    response = (wrap_argument(reponse),)
            except:
                # report exception back to server
                response = rpclib.dumps(rpclib.Fault(1, "%s:%s" % (sys.exc_type, sys.exc_value)))
                status = 500
                import traceback
                traceback.print_exc()
            else:
                response = self.dumps(method,
                                      tuple(map(wrap_argument,response)))
                status = 200
        except:
            print 'internal error', sys.exc_type, sys.exc_value
            # internal error, report as HTTP server error
            self.send_response(500)
            self.end_headers()
        else:
            # got a valid response
            self.send_response(status)
            self.send_header("Content-type", "text/xml")
            self.send_header("Content-length", str(len(response)))
            self.end_headers()
            self.wfile.write(response)

            # shut down the connection (from Skip Montanaro)
            self.wfile.flush()
            self.connection.shutdown(1)


    def call(self, method, params) :
        # RPC handler must have references to objects whose methods are shared,
        # and define the EXPORTED_METHODS dictionnary which lists the shared
        # methods and gives the objects which provide these methods.

        try:
            object = getattr(self,self.EXPORTED_METHODS[method])
            if object :
                result = apply(getattr(object,method), params)
                if result != None :
                    return (result,)
            else :
                print 'RpcService error: No',object,'to call'
        except Exception, e :
            print 'Error: %s in %s when calling method %s with params %s' \
                  % (str(e), self.__class__, method, str(params))
        except KeyError :
            print '*-'*20
            print 'RPC call to undefined/forbidden method',method,params
        return ()


if __name__ == '__main__' :
    server = SocketServer.TCPServer(('', 8000), RequestHandler)
    server.serve_forever()

