# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""ALDOMImplementation module.

Defines a DOM implementation, a factory that creates Memory and Plan documents.
"""

__revision__ = "$Id: ALDOMImplementation.py,v 1.15 2001/08/15 16:05:48 guil Exp $"

from narval.lib import AL_NS


class ALDOMImplementation:
    """Factory for memory and plan documents."""

    def __init__(self):
        self.db = None
        self.tx = None
        self.implementation = 'pDomlette'
        self.use_database = 0
        self.database_name = 'ods:alf@orion:5432:narval_db'

    def createMemoryDocument(self,al=None):
        import MemoryDocument
        if self.use_database and not self.db:
            from Ft.Ods import Database
            self.db = Database.Database()
            self.db.open(self.database_name)
        if self.use_database:
            self.tx = self.db.new()
            self.tx.begin()

        doc = MemoryDocument.MemoryDocument(al)
        h = doc.createElementNS(AL_NS,'memory')
        doc.appendChild(h)
        if self.use_database:
            if self.db.lookup("NarvalMemory"):
                self.db.unbind("NarvalMemory")
            self.db.bind(doc,"NarvalMemory")
            self.checkpointMemory()
        return doc

    def createPlanDocument(self) :
        from narval.dom.ALDomAbstraction import ALDocument
        if self.implementation == 'pDomlette':
            doc = ALDocument()
        else:
            doc = ALDocument(None)
        h = doc.createElementNS(AL_NS,'plan_document')
        doc.appendChild(h)
        return doc

    def checkpointMemory(self):
        if self.tx:
            print 'memory checkpoint'
            self.tx.checkpoint()

    def commitMemory(self,get_mem = 1):
        m = None
        if self.tx:
            print 'committing memory'
            self.tx.commit()
            self.tx = self.db.new()
            self.tx.begin()
            if get_mem:
                m = self.db.lookup('NarvalMemory')
            print 'done'
        return m

    
ALImplementation = ALDOMImplementation()
#print 'end import ALDOMImplementation'
