# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""AlDomAbstraction module.

Abstraction layer to use different DOM implementations (4DOM, pDomlette, etc.)
"""

__revision__ = "$Id: ALDomAbstraction.py,v 1.11 2001/08/10 12:14:38 alf Exp $"

from ALDOMImplementation import ALImplementation

if ALImplementation.implementation == '4DOM':
    from xml.dom.Element import Element
    from xml.dom.Document import Document
elif ALImplementation.implementation == 'pDomlette':
    from Ft.Lib.pDomlette import Element,Document
##    from narval.ext.pDomlette import Element,Document
else:
    raise 'Unknown DOM Implementation'

def myremoveAttributeNS(self,nsURI,lname):
    node = self.getAttributeNodeNS(nsURI,lname)
    if node:
        self.removeAttributeNode(node)
    return

if not hasattr(Element,'removeAttributeNS'):
    Element.removeAttributeNS = myremoveAttributeNS

def mysetAttribute(self,name,value):
    return self.setAttributeNS('',name,value)

if not hasattr(Element,'setAttribute'):
    Element.setAttribute = mysetAttribute


def _changeowner(node,owner):
    node.ownerDocument = owner
    for a in node.attributes or []:
        a.ownerDocument = owner
    [_changeowner(c,owner) for c in node.childNodes or []]

def myimportNode(self,importedNode,deep=0):
    try:
        new_node = importedNode.cloneNode(deep)
        _changeowner(new_node,self)
        return new_node
    except Exception,e:
        print importedNode.__class__
        raise e

if not hasattr(Document,'importNode'):
    Document.importNode = myimportNode


class ALElement(Element) :
    """Almost identical to Element, except for __init__() and the
    *link() bit of interface"""

    def __init__(*args) :
        if len(args)==5: # this is when called from cloneNode
            apply(Element.__init__,args)
        else: # standard call from createElementNS
            (self, ownerDoc, nodeName, namespaceURI, prefix, localName)=args
            try:
                Element.__init__(self,ownerDoc,namespaceURI,localName,prefix)
            except: # 4DOM
                Element.__init__(self,ownerDoc,nodeName,namespaceURI,prefix,localName)
        args[0].unlink()


    def get_identity(self):
        """returns the (group,name) attributes"""
        return (self.getAttributeNS('','group'),self.getAttributeNS('','name'))
        
    def __getinitargs__(self):
        return (self.ownerDocument,
                self.nodeName,
                self.namespaceURI,
                self.prefix,
                self.localName
                )

    def link(self) :
        pass

    def unlink(self) :
        pass


class ALDocument(Document) :
    """Identical to Document"""
    
    pass


