# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""Narval StyleSheet Reader module"""

__revision__ = "$Id: NarvalStylesheetReader.py,v 1.4 2001/10/09 14:54:31 syt Exp $"

from xml.xslt.StylesheetReader import StylesheetReader

from Ft.Lib.ReaderBase import StrStream
from narval.lib import element_to_string

from xml.xslt import XSL_NAMESPACE
from narval.config import get_home
from string import rfind
from os.path import exists,normpath,join
from os import sep

class NarvalStylesheetReader(StylesheetReader):
    """ Redefines a Stylesheet Reader for XSLT Processor in order to be able
    to read the stylesheets in NARVAL memory."""

    # subdirectory of NARVAL_HOME that contains the XSL Transformations
    TRANSFORMS_SUBDIRECTORY = "transforms"


    def __init__(self, force8Bit=0, memory=None, trgroup=""):
        """ memory refers to NARVAL memory. trgroup is a string containing
        the group of the main transformation (i.e. in NARVAL the
        transformation that created the associated XSLT 'Processor'). trgroup
        is used as a default group when no group is specified in the
        xsl:import and xsl:include 'href' URIs. """

        # keeps initial arguments in __initargs__
        self.__initargs__ = [force8Bit,memory,trgroup]

        self._narvalmemory = memory
        self._defaulttrgroup = trgroup
        StylesheetReader.__init__(self, force8Bit)
        return

    def __getinitargs__(self) :
        """ Returns the initial arguments used to create the instance (i.e.
        the arguments passed to __init__ . This method is used to clone
        this object (necessary to process the xsl:include). """
        return self.__initargs__
    
    def fromUri(self, uri, baseUri='', ownerDoc=None, stripElements=None,
                importIndex=0, importDepth=0):
        """ Creates a DOM node from the stylesheet at the URI 'uri'.
        The stylesheet referenced by 'uri' is first searched in NARVAL i.e. :
          - if the uri ends by a .xslt, it is searched in the memory,
          - if the uri ends by another thing, it is searched in the
              NARVAL_HOME/transforms/ directory.
        If it doesn't exist there, it is searched as a document referenced by
        a classic URI.
        The URI of a stylesheet referenced in NARVAL should be like :
        group/filename.ext with 'group' the NARVAL group of the transformation
        (i.e. the subdirectory of NARVAL_HOME/transforms where it can be
        found), and 'filename.ext' the name of the file where the
        transformation is stored (when ext='xslt' the transformation has been
        loaded in NARVAL memory, thus it is retrieved from the memory instead
        of the disk). """
        stream = None

        # Tries to get a stylesheet transform with 'uri' reference from
        # NARVAL

        # processes uri to find a group, a file name and a file extension
        temp = uri
        if rfind(temp,".") == -1 or rfind(temp,".") < rfind(temp,"/") :
            fext = ""
        else :
            fext = temp[rfind(temp,".")+1:]
            temp = temp[:rfind(temp,".")]
        if rfind(temp,"/") == -1 :
            fname = temp
            temp = ""
        else :
            fname = temp[rfind(temp,"/")+1:]
            temp = temp[:rfind(temp,"/")]
        if rfind(temp,"/") == -1 :
            group = temp
            temp = ""
        else :
            group = temp[rfind(temp,"/")+1:]
            temp = temp[:rfind(temp,"/")+1]
       
        # checks that uri is of type 'group/filename.ext' (i.e. temp is now
        # empty)
        if not(temp) :
            # if no group was defined, uses the default group
            if not (group) :
                group = self._defaulttrgroup

            if fext=="xslt" :
                # if file extension is "xslt", stylesheet must be searched in
                # NARVAL memory
                trnode = self._narvalmemory.get_transform(group+"."+fname)
                if trnode :
                    # changes trnode to the root node of the stylesheet
                    for nd in trnode.childNodes :
                        if (nd.namespaceURI == XSL_NAMESPACE) and \
                           (nd.localName in ['transform','stylesheet']) :
                            trnode = nd
                            break
                    # not very nice : creates a string from the node to be
                    # able to create a stream
                    str = element_to_string(trnode,encoding="UTF-8")
                    stream = StrStream(str)

            else :
                # if the other cases, stylesheet is searched in
                # NARVAL_HOME/transforms subdirectories
                filename = normpath(join(get_home(),
                                         self.TRANSFORMS_SUBDIRECTORY))
                if group :
                    filename = filename + sep + group
                filename = filename + sep + fname
                if fext :
                    filename = filename + "." + fext
                if exists(filename) :
                    try:
                        stream = open(filename,"r")
                    except IOError:
                        pass


        # If uri couldn't be solved in NARVAL, processes it as a classic URI
        if not(stream) :
            uri,baseUri = self.uriResolver.normalize(uri,baseUri)
            stream = self.uriResolver.resolve(uri, baseUri)

        # Reads the stylesheet from the stream
        rt = self.fromStream(stream, baseUri, ownerDoc, stripElements,
                             importIndex, importDepth)
        stream.close()
        return rt
