# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""Prototype module.

Models steps/actions/transforms prototypes.
"""

__revision__ = "$Id: Prototype.py,v 1.9 2001/08/01 16:44:26 alf Exp $"

from narval.lib import *

class PrototypeException(Exception) :
    pass


class Entry :
    """A prototype entry."""

    def __init__(self) :
        self.data = {}

    def __getitem__(self,key) :
        return self.data[key]

    def __setitem__(self,key,val) :
        self.data[key] = val

    def __str__(self) :
        return str(self.data)

        
class Prototype :
    """A step/action/transform prototype is a list of entries."""

    def __init__(self,list) :
        self.entries = []
        for entry in list :
            self.entries.append( self._make_entry(entry) )


    def add(self,entry) :
        new = self._make_entry(entry)
        for old in self.entries :
            if old['id'] == new['id'] :
                self._merge(old,new)
                break
        else :
            raise PrototypeException('ID %s not found' % (new['id']))


    def remove(self,entry):
        # removes an optional entry with no prototype
        self.entries.remove(entry)

    def _value(self,str) :
        if str == 'yes' : return 1
        elif str == 'no' : return 0
        else : return None
        
    def _make_entry(self,entry) :
        d = Entry()
        d['id'] = entry.getAttributeNS(AL_NS,'id')
        d['optional'] = self._value(entry.getAttributeNS(AL_NS,'optional'))
        d['list'] = self._value(entry.getAttributeNS(AL_NS,'list'))
        d['use'] = self._value(entry.getAttributeNS(AL_NS,'use'))
        d['outdates'] = entry.getAttributeNS(AL_NS,'outdates')
        d['match'] = []
        for t in entry.childNodes:
            if t.nodeName == 'match' and t.firstChild:
                d['match'].append(t.firstChild.data)
        return d

        
    def _merge(self,old,new) :
        if old['optional'] and new['optional'] == 0 :
            old['optional'] = 0
        if old['list'] and new['list'] == 0 :
            old['list'] = 0
        if not old['use'] and new['use'] == 1 :
            old['use'] = 1
        if not old['outdates'] and new['outdates'] :
            old['outdates'] = new['outdates']
        for m in new['match'] :
            if m not in old['match'] :
                old['match'].append(m)
        
