# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""Tags module.

Handles relationships between elements.

Steps and transitions can tag elements to mark that they used them. Later,
steps and transitions that have used='no' in their prototype will have
elements tagged by them filtered out automatically.
"""

from narval.lib import AL_NS

from Ft.Rdf.Model import Model
from Ft.Rdf.Drivers.Memory import DbAdapter
from Ft.Rdf.Statement import Statement

TagsDb = DbAdapter('narval_tags')
TagsModel = Model(TagsDb)


def _get_id(element) :
    return element.getAttributeNS(AL_NS,'eid') or str(hash(element))


def _tagger_id(node) :
    """node should be a step or a transition. id is 'plan_name.step_id'
    """
    return '.'.join((node.parentNode.getAttributeNS(AL_NS,'name'),
                     node.getAttributeNS(AL_NS,'id')))

def is_tagged(element,node) :
    TagsDb.begin()
    answer = TagsModel.complete(_get_id(element),'tagged-by',_tagger_id(node))
    TagsDb.commit()
    return answer


def tag_element(element,node) :
    s = Statement(_get_id(element),'tagged-by',_tagger_id(node))
    TagsDb.begin()
    TagsModel.add([s])
    TagsDb.commit()


def clear_tags(element):
    TagsDb.begin()
    TagsModel.removePattern(_get_id(element),'tagged-by',None)
    TagsDb.commit()


def fromtag(element,node) :
    element.setAttributeNS(AL_NS,'from_plan',
                           node.parentNode.getAttributeNS(AL_NS,'eid'))
    element.setAttributeNS(AL_NS,'from_step', node.getAttributeNS(AL_NS,'id'))

