# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""TransformElement"""

__revision__ = "$Id: TransformElement.py,v 1.33 2001/10/02 15:30:56 syt Exp $"

from ALDomAbstraction import ALElement
from Prototype import Prototype
from NarvalStylesheetReader import NarvalStylesheetReader

from narval.lib import *
from narval.bibal import *

from xml.xslt.DomWriter import DomWriter
from xml.xslt.Processor import Processor
from xml.xslt import XSL_NAMESPACE, XsltException
import xml.xpath.Util
import copy

class TransformElement(ALElement) :
    """TransformElement XXX"""

    # Element methods overriden ################################################

    def __repr__(self):
        return "<TransformElement Node at %s: %d attributes and %d children>" %\
               ( id(self), len(self.attributes), len(self.childNodes) )

    # ALElement init interface #################################################

    def unlink(self) :
        self.inputs = {}
        self.outputs = {}
        self.processor = None
        self.init = false
        
    def link(self) :
        if self.init : return
        for proto in self.firstChild.childNodes:
            if proto.nodeName == 'prototype':
                break
        
        for node in proto.childNodes:
            name = node.nodeName
            # input
            if name == 'input':
                inputs = []
                self.inputs[node] = inputs
                for t in node.childNodes:
                    if t.nodeName == 'match':
                        inputs.append(t.firstChild.data)
            # outputs
            elif name == 'output':
                outputs = []
                self.outputs[node] = outputs
                for t in node.childNodes:
                    if t.nodeName == 'match':
                        outputs.append(t.firstChild.data)
            
        self.init = true

    # Transform element methods ################################################

    def get_inputs(self) :
        self.link()
        return Prototype(self.inputs.keys())

    def get_outputs(self) :
        self.link()
        return Prototype(self.outputs.keys())
    
    def mk_processor(self) :
        if self.processor : return
        # builds a new processor
        self.processor = Processor()
        self.register_extensions()
        # with a NarvalStylesheetReader
        stsheetreader = NarvalStylesheetReader\
                        (memory = self.ownerDocument,\
                         trgroup = self.getAttributeNS('','group'))
        self.processor.setStylesheetReader(stsheetreader)
        
        for node in self.childNodes :
            if (node.namespaceURI == XSL_NAMESPACE) and \
               (node.localName in ['transform','stylesheet']) :
                s = element_to_string(node,encoding="UTF-8")
                xpath_lock.acquire()
                try:
                    self.processor.appendStylesheetString(s)
                except XsltException, e:
                    print 'xslt error in', s
                    raise e
                xpath_lock.release()
                break

    def register_extensions(self, modules_list = None):
        import os
        import string
        modules_list = modules_list or []
        narval_ext_path = config.get_home()+'/transforms/extensions'
        if narval_ext_path not in sys.path:
            sys.path.append(narval_ext_path)
        for entry in os.listdir(narval_ext_path):
            if not os.path.isdir(entry) and entry[-3:] == '.py' and entry != '__init__.py':
                modules_list.append(entry[:-3])
                
        env = os.environ.get('EXTMODULES')
        if env is None:
            if len(modules_list):
                self.processor.registerExtensionModules(modules_list)
            return
        if os.name == 'posix':
            dir_list = string.split(env,':')
        elif os.name in ['nt','dos']:
            dir_list = string.split(env,';')
        else:
            if len(modules_list):
                self.processor.registerExtensionModules(modules_list)
            return
        for dir in dir_list:
            for entry in os.listdir(dir):
                if not os.path.isdir(entry) and entry[-3:] == '.py' and entry != '__init__.py':
                    modules_list.append(entry[:-3])
        if len(modules_list):
            self.processor.registerExtensionModules(modules_list)
        
    def transform(self,element,target_doc) :
        self.mk_processor()
        xpath_lock.acquire()
        xml.xpath.Util.FreeDocumentIndex(element.ownerDocument)
        xml.xpath.Util.IndexDocument(element.ownerDocument)
        dom = self.processor.runNode(element,1,{},DomWriter(target_doc))
        xpath_lock.release()
        for node in dom.childNodes :
            node.normalize() # normalize doesn't work on document fragments
        dom = StripXml(dom)
        return dom.childNodes
