# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""This code is a temporary replacement for 4DOM xml/dom/ext/reader/HtmlLib.py

If 4Thought accepts the present replacement and integrates it to 4DOM, this
module will disappear from Narval's distribution.

Most of this code comes from xml/dom/ext/reader/HtmlLib.py
"""

__revision__ = "$Id: HtmlLib.py,v 1.8 2001/06/15 14:30:38 adim Exp $"

import string
from sgmllib import SGMLParser
from xml.dom import implementation

from HTMLDTD import HTMLDTD

class HtmlToDomParser(SGMLParser):
    """Html to Dom parser"""
    
    def __init__(self):
        SGMLParser.__init__(self,1)
        self.stack = []
        self.ownerDoc = None

    def unknown_starttag(self,tag,attrs):
        tag = string.upper(tag)
        # create element and attributes
        newElement = None
        try:
            newElement = self.ownerDoc.createElement(tag)
        except TypeError :
            return
        for attr in attrs:
            name,value = attr
            newElement.setAttributeNS('',name,value)

        # look for its parent
        for i in range(1,len(self.stack)) :
            t = self.stack[-i]
            if string.lower(tag) in HTMLDTD[string.lower(t.tagName)] :
                t.appendChild(newElement)
                if i > 1 :
                    self.stack = self.stack[:-i+1]
                if HTMLDTD[string.lower(tag)] :
                    self.stack.append(newElement)
                return
            
        # no parent found
        if tag == 'HTML' :
            self.stack[0].appendChild(newElement)
            self.stack.append(newElement)
        else :
            #print 'NO PARENT FOR',tag,'IN STACK',self.stack
            pass # -- DISCARD element

    def unknown_endtag(self, tag):
        tag = string.upper(tag)

        # look for opening tag
        for i in range(1,len(self.stack)) :
            t = self.stack[-i]
            if tag == t.tagName :
                self.stack = self.stack[:-i]
                return

        # no opening tag found
        #print 'NO OPENING TAG FOR',tag # -- DISCARD element

    def handle_data(self, data):
        t = self.ownerDoc.createTextNode(data)
        self.handle_generic_node(t)

    def handle_comment(self, comment):
        c = self.ownerDoc.createComment(comment)
        self.handle_generic_node(c)

    def handle_generic_node(self, node):
        self.stack[-1].appendChild(node)

    #def report_unbalanced(self, tag):
    #    print "Unbalanced tag"

    def toDom(self, st, ownerDoc=None):
        self.ownerDoc = ownerDoc or implementation.createHTMLDocument('') ### FIXME : use right implementation
        # Parse everythin into a DF
        self.stack = [self.ownerDoc.createDocumentFragment()]
        self.feed(st)
        self.close()
        return self.stack[0]


def FromHtmlStream(fp, ownerDocument=None):
    return FromHtml(fp.read(), ownerDocument)


def FromHtmlFile(fileName, ownerDocument=None):
    f = open(fileName,'r')
    rv = FromHtmlStream(f,ownerDocument)
    f.close()
    return rv


def FromHtmlUrl(url, ownerDocument=None):
    import urllib
    f = urllib.urlopen(url)
    rv = FromHtmlStream(f,ownerDocument)
    f.close()
    return rv


def FromHtml(str, ownerDocument=None):
    p = HtmlToDomParser()
    d = p.toDom(str,ownerDocument)

    #d is a DF
    #if ownerDocument != None or DF.childNodes() has a HTML tag put it in a document
    toDoc = None
    if ownerDocument != None:
        toDoc = ownerDocument
    else:
        children = d.childNodes
        for child in children:
            if child.nodeType == child.ELEMENT_NODE and child.tagName == 'HTML':
                toDoc = p.ownerDoc
                break
    if toDoc:
        from xml.dom.ext import PrettyPrint
        #Convert to a document
        for child in d.childNodes:
            if child.nodeType == child.ELEMENT_NODE and child.tagName == 'HTML':
                newNode = toDoc.importNode(child, 1)
                toDoc.replaceChild(newNode, toDoc.documentElement)
            elif child.nodeType != child.TEXT_NODE:
                toDoc.appendChild(child)
        d = toDoc
    return d


if __name__ == '__main__' :
    import sys
    d = FromHtmlFile(sys.argv[1])
    from xml.dom.ext import PrettyPrint
    PrettyPrint(d)
