# Copyright (c) 2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307
# USA.
__revision__ = "$Id: HornActioner.py,v 1.27 2001/10/10 11:06:09 syt Exp $"

from NarvalProxy import *
from NpmManager import *
from HornModel import *
from MemoryModel import *
from HornRPC import HornRpcHandler
from MemoryController import MemoryController
from EngineController import EngineController

from narval.Engine import InteractionInterface
from narval.Engine import DebugInterface
from narval.lib import FALSE,TRUE

from narval.lib import select_many
RPC_LISTENER_PORT = 9513

from threading import *
from narval.gui import unicode_to_gtk

class CommonActioner:
    """ common actioner to  cli and gui """

    _DICT_ = {}

    def __init__(self):
        self.__dict__ = self._DICT_

        if not self.__dict__ :
            self._npm = NpmManager()
            self._narval_proxy = NarvalProxy()
            # list of listeners
            self.listeners = {'plan':[],'memory':[],'engine':[]}
            # time_id
            self.time_id = None

    def start_local(self):
        self.disconnect_from_narval()
        MemoryModel().loadLocalData()

    ## connexions functions ###################################################
        
    def connect_to_narval(self,host,port):
        hmodel = HornModel()
        if hmodel.narval :
            print "Already connected to Narval"
            return FALSE
        else:
            import socket
            self._setup_local_rpc_server(RPC_LISTENER_PORT)
            try:
                from narval import RpcServer
                hmodel.set_narval(RpcServer.Server("http://%s:%d"%(host,port)))
                host = socket.gethostname()

                hmodel.narval.addRemoteMemoryListener(host,self._rpc_port)
                hmodel.narval.addRemotePlanListener(host,self._rpc_port)
                hmodel.narval.addRemoteEngineListener(host,self._rpc_port)
                
                # create a new MemoryController link to narval memory
                mem_ctrl = MemoryController()
                self.listeners['memory'].append(mem_ctrl)
                self.listeners['plan'].append(mem_ctrl)

                # create an EngineController link to narval
                eng_ctrl = EngineController()
                self.listeners['memory'].append(eng_ctrl)
                self.listeners['plan'].append(eng_ctrl)
                self.listeners['engine'].append(eng_ctrl)
                
                # load narval memory
                MemoryModel().loadNarvalMemory(NarvalProxy().getElementsList())
                return TRUE
                
            except socket.error :
                self._terminate_local_rpc_server()
                # notify that narval is not connected
                hmodel.set_narval(None)
                return FALSE

    def disconnect_from_narval(self):
        hmodel = HornModel()
        EngineController().close_logfile()
        self.server_state = FALSE
        if hmodel.narval:
            import socket
            try:
                hmodel.narval.removeRemoteMemoryListener(socket.gethostname(),self._rpc_port)
                hmodel.narval.removeRemotePlanListener(socket.gethostname(),self._rpc_port)
                hmodel.narval.removeRemoteEngineListener(socket.gethostname(),self._rpc_port)
                
                # break link between narval memory and memory controller
                mem_ctrl = MemoryController()
                _remove_listener(self.listeners, 'memory', mem_ctrl)
                _remove_listener(self.listeners, 'plan', mem_ctrl)

                # break link between narval memory and engine controller
                eng_ctrl = EngineController()
                _remove_listener(self.listeners, 'memory', eng_ctrl)
                _remove_listener(self.listeners, 'plan', eng_ctrl)
                _remove_listener(self.listeners, 'engine', eng_ctrl)

                # FIXME update memory...
                self._terminate_local_rpc_server()
            except:
                raise
            hmodel.set_narval(None)

    ## install / uninstall ####################################################
    def install(self,file):
        self._npm.install(file)
    def uninstall(self,file):
        self._npm.uninstall(file)
    def list_packages(self) :
        return self._npm.list_packages()
        
    ## implement DebugInterface ###############################################
    def debugSuspend(self):
        self._narval_proxy.debugSuspend()
    def debugContinue(self):
        self._narval_proxy.debugContinue()
    def debugStepOneStep(self):
        self._narval_proxy.debugStepOneStep()

    ## Interface that defines Engine methods used by (Graph|CmdLine)UI ########
    def getMemory(self):
        return MemoryModel().get_memory()        
    def getElement(self,id):
        return self._narval_proxy.getElement(id)
    def getElementsList(self):
        return self._narval_proxy.getElementsList()
    def getElementList(self,id):
        return self._narval_proxy.getElementList(id)
    def selectElements(self,xpath):
        l = select_many(MemoryModel().get_memory(), xpath)
        from cStringIO import StringIO
        s = StringIO()
        for elt in l:
            s.write(element_to_string(elt)+"\n")
        return s.getvalue()
    def addElement(self,elementXml):
        self._narval_proxy.addElement(elementXml)
    def removeElement(self,elementId):
        self._narval_proxy.removeElement(elementId)
    def replaceElement(self,elementId,elementXml):
        self._narval_proxy.replaceElement(elementId,elementXml)
    def startPlan(self,recipeName):
        self._narval_proxy.startPlan(recipeName)
    def saveRecipes(self):
        self._narval_proxy.saveRecipes()
    def saveTransforms(self):
        self._narval_proxy.saveTransforms()
    def shutdown(self):
        self._narval_proxy.shutdown()

    ## rpc callbacks ##########################################################
    def handle_incoming_rpc_request(self,socket,condition):
        import select
        try:	
            fno = socket.fileno()
        except:
            pass # Ugly way of fixing nasty bug on windows 	
	else:
	    if fno == -1:
		print "Warning: bad socket in mainloop"
	    else:
		lr,lw,le = select.select([socket],[],[socket],0.01)
		if lr:
		    data = socket.accept()
		    HornRpcHandler().start(data[0],data[1],None)
		elif le:
		    print 'error message on RPC socket'
		    print socket.recv()

    def process_queue(self):
        import Queue
        try:
            event = HornModel().queue.get_nowait()
            objects = self.listeners[event[0]]
            for object in objects:
                if object: 
                    apply(getattr(object,event[1]),event[2])
        except Queue.Empty :
            pass
        return TRUE

    ## PRIVATE ################################################################
    def _setup_local_rpc_server(self,port):
        import socket
        self.rpc_socket = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        self.rpc_socket.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR,1)
        #self.rpc_socket.setblocking(0)
        notconnected = true
        while notconnected :
            try:
                self.rpc_socket.bind(('',port))
            except:
                port = port + 1
            else:
                notconnected = false
        print 'RPC server created on port',port
        self._rpc_port = port
        self.rpc_socket.listen(5)
        
    def _terminate_local_rpc_server(self):
        self.rpc_socket.close()


            
class HornActioner(CommonActioner):
    """
    Lower class linking HornModel, RecipeEditorController,
    NarvalProxy and NpmManager.
    This class is written for gui using.
    """

    def __init__(self):
        self.__dict__ = CommonActioner._DICT_
        if not self.__dict__ :
            CommonActioner.__init__(self)
        
    def connect_to_narval(self, host, port):
        if CommonActioner.connect_to_narval(self, host, port):
            # start process queue
            from gtk import timeout_add
            self.time_id = timeout_add(500,self.process_queue)
            return TRUE

    def disconnect_from_narval(self):
        hmodel = HornModel()
        if hmodel.narval:
            # stop process queue
            from gtk import timeout_remove
            timeout_remove(self.time_id)
        CommonActioner.disconnect_from_narval(self)

    ## PRIVATE ################################################################
    def _setup_local_rpc_server(self, port):
        CommonActioner._setup_local_rpc_server(self, port)
        from gtk import input_add
        import GDK
        self._handler_id = input_add(self.rpc_socket,GDK.INPUT_READ,self.handle_incoming_rpc_request)

    def _terminate_local_rpc_server(self):
        CommonActioner._terminate_local_rpc_server(self)
        from gtk import input_remove
        input_remove(self._handler_id)

    

class HornActionerCli(CommonActioner):
    """
    Horn Actioner Cli : class used by HornCli
    class use Thread object to discuss with Narval
    """

    def __init__(self):
        self.__dict__ = CommonActioner._DICT_
        if not self.__dict__: 
            CommonActioner.__init__(self)
            # need to create a thread object
            self.thread = None
            # counter index for cli recipe edition
            self._count = 0
            # local server state
            self.server_state = FALSE

    # CLI interaction #############################################

    def add_element_to_recipe(self,name,element,type,id=None):
        import string
        name = string.split(name,".")
        try:
            node = MemoryModel().select('recipe', name[0], name[1])
            if type == 'transition':
                trans = node.ownerDocument.createElementNS(AL_NS,'transition')
                if not id:
                    self._count = self._count + 1
                    id = "trans_"+str(self._count)
                trans.setAttributeNS(AL_NS,'id',id)
                trans.setAttributeNS(AL_NS,'id',id)
                node.appendChild(trans)
            else:
                step = node.ownerDocument.createElementNS(AL_NS,'step')
                step.setAttributeNS(AL_NS,'type',type)
                step.setAttributeNS(AL_NS,'target',element)
                if not id:
                    self._count = self._count + 1
                    id = "step_"+str(self._count)
                step.setAttributeNS(AL_NS,'id',id)
                node.appendChild(step)
        except:
            import traceback
            traceback.print_exc()
            print unicode_to_gtk(_('add element recipe is impossible'))

    def add_argument_to_step(self,name,id_step,argument):
        import string
        name = string.split(name,'.')
        try:
            node = MemoryModel().select('recipe', name[0], name[1])
            step = select_one(node,'step[@id="%s"]'%(id_step))
            arg = read_domtree_from_string(argument)
            step.appendChild(arg)
        except:
            print unicode_to_gtk(_('add argument is impossbile'))

    def add_attribute_to_recipe(self,recipe,tag,attr):
        import string
        name = string.split(recipe,'.')
        try:
            node = MemoryModel().select('recipe', name[0], name[1])
            node.setAttributeNS(AL_NS,tag,attr)
        except:
            import traceback
            traceback.print_exc()
            print unicode_to_gtk(_('add an attribute to recipe is impossible'))

    def add_condition_to_transition(self,recipe,id_trans,cond):
        import string
        name = string.split(recipe,".")
        try:
            node = MemoryModel().select('recipe', name[0], name[1])
            node = select_one(node, 'transition[@id="%s"]' % (id_trans))
            condition = read_domtree_from_string(cond)
            node.appendChild(condition)
        except:
            import traceback
            traceback.print_exc()
            print unicode_to_gtk(_('add condition is impossible'))

    def connect_elements(self,recipe,elt1,elt2):
        import string
        name = string.split(recipe,'.')
        try:
            node = MemoryModel().select('recipe', name[0], name[1])
            elt1_node = select_one(node,'*[@id="%s"]'%(elt1))
            elt2_node = select_one(node,'*[@id="%s"]'%(elt2))
            if elt1_node.tagName == 'step' and elt2_node.tagName == 'transition':
                in_node = elt2_node.ownerDocument.createElementNS(AL_NS,'in')
                in_node.setAttributeNS(AL_NS,'idref',elt1_node.getAttributeNS(AL_NS,'id'))
                elt2_node.appendChild(in_node)
            elif elt1_node.tagName == 'transition' and elt2_node.tagName == 'step':
                out_node = elt1_node.ownerDocument.createElementNS(AL_NS,'out')
                out_node.setAttributeNS(AL_NS,'idref',elt2_node.getAttributeNS(AL_NS,'id'))
                elt1_node.appendChild(out_node)
        except:
            import traceback
            traceback.print_exc()
            print unicode_to_gtk(_('connect elements is impossible'))

    def add_new_recipe(self, recipe, cookbook):
        MemoryController().add_recipe_to_cookbook(recipe, cookbook)

    def remove_element_to_recipe(self,recipe,id_elt):
        import string
        name = string.split(recipe,'.')
        try:
            node = MemoryModel().select('recipe', name[0], name[1])
            elt_node = select_one(node, '*[@id="%s"]' % (id_elt))
            # find connections before and destroy them
            if elt_node.tagName == 'step':
                transitions = select_many(elt_node.parentNode,'transition')
                for transition in transitions:
                    links = select_many(transition,'*[idref="%s"]'%(id_elt))
                    for link in links:
                        transition.removeChild(link)
            elt_node.parentNode.removeChild(elt_node)
        except:
            import traceback
            traceback.print_exc()
            print unicode_to_gtk(_('remove element is impossible'))

    def remove_connection_to_recipe(self,recipe,id_elt1,id_elt2):
        import string
        name = string.split(recipe,'.')
        try:
            node = MemoryModel().select('recipe', name[0], name[1])
            elt_node = select_one(node, 'transition[id="%s"]'%(id_elt1))
            link = select_one(elt_node,'*[@idref="%s"]'%(id_elt2))
            elt_node.removeChild(link)
        except:
            import traceback
            traceback.print_exc()
            print unicode_to_gtk(_('remove connection is impossible'))

    def disconnect_from_narval(self):
        self.server_state = FALSE
        CommonActioner.disconnect_from_narval(self)

    ## PRIVATE ################################################################
    def _setup_local_rpc_server(self,port):
        CommonActioner._setup_local_rpc_server(self, port)
        self.server_state = TRUE
        self.thread = ActionerThread(self)
        self.thread.start()

    def _terminate_local_rpc_server(self):
        self.server_state = FALSE
        self.thread = None
        CommonActioner._terminate_local_rpc_server(self)



class ActionerThread(Thread):
    """
    Subclass of Thread used to write th run function
    """
    
    def __init__(self,actioner):
        Thread.__init__(self)
        self._actioner = actioner

    # run function of Thread Object
    def run(self):
        while self._actioner.server_state: 
            self._actioner.handle_incoming_rpc_request(self._actioner.rpc_socket,0)
            self._actioner.process_queue()
        return



def _remove_listener(listeners, type, inst):
    # since we use borg pattern, this stuff is necessary to remove listeners
    for l in listeners[type]:
        if l.__dict__ is inst.__dict__:
            listeners[type].remove(l)
