"""
Memory Model using the borg pattern
"""
# Copyright (c) 2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307
# USA.
__revision__ = "$Id: MemoryModel.py,v 1.41 2001/10/13 08:59:24 alf Exp $"

import os,sys
from os.path import join
from narval.lib import *
from narval.bibal import loadRecipes, loadModules, loadTransforms
from narval.gui import unicode_to_gtk
# We need 4DOM in Horn
from narval.dom.ALDOMImplementation import ALImplementation
ALImplementation.implementation = '4DOM'

from Observer import Observable

_E_TYPE = 0
_E_GROUP = 1
_E_NAME = 2
_E_DOM = 3

E_ID = 0
E_TYPE = 1
E_GROUP = 2
E_NAME = 3

class MemoryModel(Observable):
    """
    the Model of the Memory
    Memory can be local or distant narval's memory
    """

    _DICT_ = {}

    def __init__(self):
        self.__dict__ = self._DICT_
        if not self.__dict__ :
            Observable.__init__(self)
            self._memory = ALImplementation.createMemoryDocument()
            self._current_step = None
            self._current_transition = None
            self._current_plan = None
            # listeners for the queue
            self.listeners = {}
            self.full_memory = FALSE
            self.eid_count = 0

            self._empty_memory()

    # initial memory load functions ###########################################

    def loadLocalData(self):
        # empty memory first...
        self._empty_memory()
        #... and fill it
        loadRecipes(self._memory,self._local_add)
        loadModules(self._memory,self._local_add)
        loadTransforms(self._memory,self._local_add)
        print unicode_to_gtk(_('*** local data has been loaded in memory ***'))
        self.notify_observers()

    def loadNarvalMemory(self,elts_list):
        # empty memory first...
        self._empty_memory()
        if elts_list:
            print unicode_to_gtk(_("Retrieving Narval's memory..."))
            # ... and fill it
            self._fill_memory(elts_list)
        self.notify_observers()

    def get_memory(self):
        if not self.full_memory:
            for id in self.eid_d.keys():
                if not self.eid_d[id][_E_DOM]:
                    #print 'GET ELEMENT FROM NARVAL', id
                    from HornActioner import HornActioner
                    xml = HornActioner().getElement(id)
                    node = read_domtree_from_string(xml,self._memory).firstChild
                    self._memory.documentElement.appendChild(node)
                    self.eid_d[id][_E_DOM] = node
            self.full_memory = TRUE
        return self._memory
    
    ## memory functions ########################################################
    def add_element(self,e_list):
        if e_list:
            id, type, group, name = e_list
            self.eid_d[id] = [type, group, name, None]
            #print 'MMODEL ADD', id, type, group, name
            if type in ('recipe', 'action', 'transform', 'plan') \
               and not self._has_key(type, group, name):
                self.base_type_d[type].setdefault(group, {})[name] = id
            else:
                self.type_d.setdefault(type, []).append(id)
            self.full_memory = FALSE
            self.changes.append(('add', e_list))
            self.notify_observers()
            

    def remove_element(self,eid):
        type, group, name, dom = self.eid_d[eid]
        #print 'MMODEL REM', eid, type, group, name, dom
        del self.eid_d[eid]
        if type in ('recipe', 'action', 'transform', 'plan') and \
           eid == self.base_type_d[type][group][name]:
            del self.base_type_d[type][group][name]
        else:
            self.type_d[type].remove(eid)
        if dom:
            dom.parentNode.removeChild(dom)
            release_domtree(dom)
        self.changes.append(('rem', [eid, type, group, name]))
        self.notify_observers()

    ## messages received from Narval because we're a MemoryListener ############
    def step_changed(self, planId, stepId, status):
        # ignore changes if not plan in local memory
        plan = self._select_local_by_id(planId)
        if plan:
            try:
                step = select_one(plan,'step[@id="%s"]' % stepId)
                step.setAttributeNS(AL_NS,'state',status)
                self.changes.append(('change', 'step', planId, step))
                self.notify_observers()
            except:
                import traceback
                traceback.print_exc()
                print unicode_to_gtk(_("step %s not found in plan")
                                     %(stepId,planId))

    def transition_changed(self,planId,transitionId,status):
        # ignore changes if not plan in local memory
        plan = self._select_local_by_id(planId)
        if plan:
            try:
                transition = select_one(plan, 'transition[@id="%s"]'
                                        % (transitionId))
                transition.setAttributeNS(AL_NS,'state',status)
                self.changes.append(('change', 'transition',planId,transition))
                self.notify_observers()
            except:
                import traceback
                traceback.print_exc()
                print unicode_to_gtk(_("transition %s not found in plan")
                                     %(transitionId,planId))

    def plan_changed(self,planId,status):
        # ignore changes if not plan in local memory
        plan = self._select_local_by_id(planId)
        if plan:
            try:
                plan.setAttributeNS(AL_NS,'state',status)
                self.changes.append(('change', 'plan', planId,plan))
                self.notify_observers()
            except:
                import traceback
                traceback.print_exc()
                print unicode_to_gtk(_('plan %s not found') % planId)

    
    # Cookbook methods ########################################################
    def get_cookbooks(self):
        return self.base_type_d['recipe'].keys()

    def add_recipe_to_cookbook(self,name,cookbook):
        if name and cookbook:
            self.eid_count += 1
            id = unicode_to_gtk(str(self.eid_count))
            recipe = self._memory.createElementNS(AL_NS,'recipe')
            recipe.setAttributeNS(AL_NS,'name',name)
            recipe.setAttributeNS(AL_NS,'group',cookbook)
            recipe.setAttributeNS(AL_NS,'eid', id)
            self._memory.documentElement.appendChild(recipe)
            self.eid_d[id] = ['recipe', cookbook, name, recipe]
            self.base_type_d['recipe'].setdefault(cookbook, {})[name] = id
            self.changes.append(('add', [id, 'recipe', cookbook, name]))
            self.notify_observers()

    def get_xmls_cookbook(self, name):
        """get recipes from cookbook as xml string"""
        from StringIO import StringIO
        s = StringIO()
        s.write('<?xml version="1.0" encoding="UTF-8"?>\n<cookbook name="%s">'%name)
        for recipe in self.base_type_d['recipe'][name].keys():
            id = self.base_type_d['recipe'][name][recipe]
            PrettyPrint(self.eid_d[id][_E_DOM], s, 'UTF-8')
        s.write("</cookbook>")
        xmls = s.getvalue()
        s.close()
        return xmls


    # recipe methods ##########################################################
    def save_recipe(self, recipe):
        # WARNING: recipe is a dom tree, not an eid
        try:
            from HornModel import HornModel
            if HornModel().narval:
                from HornActioner import HornActioner
                HornActioner().replaceElement(recipe.getAttributeNS(AL_NS,'eid'),
                                              element_to_string(recipe))
            else:
                from MemoryController import MemoryController
                id = recipe.getAttributeNS(AL_NS,'eid')
                tmprecipe = self.select_by_id(id)
                r = tmprecipe.parentNode.ownerDocument.importNode(recipe,DEEP_COPY)
                tmprecipe.parentNode.replaceChild(r,tmprecipe)
                self.eid_d[id][_E_DOM] = r
                MemoryController().save_cookbook(recipe.getAttributeNS(AL_NS,'group'))
        except:
            import traceback
            traceback.print_exc()

    def remove_recipe(self, eid):
        from HornModel import HornModel
        if HornModel().narval:
            from HornActioner import HornActioner
            HornActioner().removeElement(eid)
        else:
            try:
                tmprecipe = select_by_id(eid)
                tmprecipe.parentNode.removeChild(tmprecipe)
            except:
                print unicode_to_gtk(_('problem when recipe is removed')),recipe


    # public nodes selection methods ###########################################
    # those functions suppose we can't have a request for a 'not in memory' node
    def select_by_id(self, id):
        # do we ever have this element ?
        if self.eid_d[id][_E_DOM]:
            return self.eid_d[id][_E_DOM]
        else:
            #print 'GET ELEMENT FROM NARVAL', id
            from HornActioner import HornActioner
            xml = HornActioner().getElement(id)
            if xml:
                node = read_domtree_from_string(xml,self._memory).firstChild
                self._memory.documentElement.appendChild(node)
                self.eid_d[id][_E_DOM] = node
                return node
            else:
                print 'Element %d not found in memory' % id
            return node

    def select(self, type, group, name=None):
        assert type in ('transform', 'action', 'recipe', 'plan')
        # do we ever have this element ?
        if name:
            return self.select_by_id(self.base_type_d[type][group][name])
        elif group:
            l = []
            for name in self.base_type_d[type][group].keys():
                l.append(self.select_by_id(self.base_type_d[type][group][name]))
                
    def get_elements_list(self, type=None):
        l = []
        if not type:
            for type in ('recipe', 'action', 'transform', 'plan'):
                l += self.get_elements_list(type)
            for type in self.type_d.keys():
                if not type in ('recipe', 'action', 'transform', 'plan'):
                    l += self.get_elements_list(type)
        else:
            if type in ('recipe', 'action', 'transform', 'plan'):
                for group in self.base_type_d[type].keys():
                    for name in self.base_type_d[type][group].keys():
                        l.append((self.base_type_d[type][group][name],
                                  type, group, name))
                if self.type_d.has_key(type):
                    for id in self.type_d[type]:
                        # FIXME: element may have group/name attribute
                        l.append((id, type, '', ''))
            else:
                for id in self.type_d[type]:
                    # FIXME: element may have group/name attribute
                    l.append((id, type, '', ''))
        return l
    
    def get_element_list(self, id):
        return [id] + self.eid_d[id][:-1]
    
    def get_eid(self, type, group, name):
        assert type in ('transform', 'action', 'recipe', 'plan')
        return self.base_type_d[type][group][name]

    
    ## PRIVATE ################################################################
    def _has_key(self, type, group, name):
        assert type in ('transform', 'action', 'recipe', 'plan')
        try:
            self.base_type_d[type][group][name]
            return TRUE
        except:
            return FALSE
            
    def _select_local_by_id(self, id):
        if self.eid_d[id][_E_DOM]:
            return self.eid_d[id][_E_DOM]
        else: return None

    def _fill_memory(self,elts_list):
        for id, type, group, name in elts_list:
            self.eid_d[id] = [type, group, name, None]
            if type in ('recipe', 'action', 'transform', 'plan') and \
               not self._has_key(type, group, name):
                self.base_type_d[type].setdefault(group, {})[name] = id
            else:
                self.type_d.setdefault(type, []).append(id)
                    
    def _empty_memory(self) :
        release_domtree(self._memory)
        self._memory = ALImplementation.createMemoryDocument()
        self.eid_d = {}
        self.base_type_d = {'recipe':{}, 'transform':{}, 'action':{}, 'plan':{}}
        # for all others elements
        self.type_d = {}

    def _local_add(self, node):
        self.eid_count += 1
        eid = str(self.eid_count)
        node.setAttributeNS(AL_NS,'eid', eid)
        self._memory.documentElement.appendChild(node)
        type = unicode_to_gtk(node.nodeName)
        name = unicode_to_gtk(node.getAttributeNS(AL_NS,'name'))
        group = unicode_to_gtk(node.getAttributeNS(AL_NS,'group'))
        eid = unicode_to_gtk(eid)
        self.eid_d[eid] = [type, group, name, node]
        self.base_type_d[type].setdefault(group, {})[name] = eid
        
