# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
__revision__ = "$Id: PrefController.py,v 1.6 2001/10/08 16:13:45 syt Exp $"

from PrefModel import PrefModel
from gtk import *
from xml.xpath import Evaluate
from xml.dom.ext import Print,PrettyPrint

from narval.lib import *
from narval.gui import unicode_to_gtk

class PrefController:
    """
    Controller of user preferences using singleton pattern
    """

    _DICT_ = {}

    def __init__(self):
        self.__dict__ = self._DICT_

        if not self.__dict__ :
            # intialisation
            self._type = 'step'
            self._state_step = ''
            self._state_transition = ''

    ### PUBLIC ###

    def save_preferences(self):
        PrefModel().save_preferences()

    def default(self):
        self._state_step = PrefModel().get_list_step_style().keys()[0]
        self._state_transition = PrefModel().get_list_transition_style().keys()[0]
        PrefModel().update_sample_from_menu(self._type,self._state_step)

    # style callbacks #

    def type_selection_changed(self,*args):
        self._type = args[0].get_data('type')
        args[3].remove_menu()
        if self._type == 'step':
            args[3].set_menu(args[1])
            args[3].show_all()
            PrefModel().update_sample_from_menu(self._type,self._state_step)
        else:
            args[3].set_menu(args[2])            
            args[3].show_all()
            PrefModel().update_sample_from_menu(self._type,self._state_transition)

    # state selection callback

    def state_selection_changed(self,*args):
        if self._type == 'step':
            self._state_step = args[0].get_data('state')
            PrefModel().update_sample_from_menu(self._type,self._state_step)
        else:
            self._state_transition = args[0].get_data('state')
            PrefModel().update_sample_from_menu(self._type,self._state_transition)

    # choose color callbacks #

    def choose_color(self,btn,key):
        import string
        if key == 'font-color':
            cc = PrefModel().get_current_style().fg[STATE_NORMAL]
            title = unicode_to_gtk(_("Font color"))
        elif key == 'border-color':
            cc =  PrefModel().get_current_style().dark[STATE_NORMAL]
            title = unicode_to_gtk(_("Border color"))
        elif key == 'background':
            cc = PrefModel().get_current_style().bg[STATE_NORMAL]
            title = unicode_to_gtk(_("Background color"))
        elif key == 'popped-color':
            cc = PrefModel().get_popped_color()
            title = unicode_to_gtk(_("Popped color"))
        elif key == 'pushed-color':
            cc = PrefModel().get_pushed_color()
            title = unicode_to_gtk(_("Pushed color"))
        else:
            cc = None

        cs = GtkColorSelectionDialog(title)

        color_channel = (float(cc.red)/65535,float(cc.green)/65535,float(cc.blue)/65535,-1)
        cs.colorsel.set_color(color_channel)
        cs.set_position(WIN_POS_MOUSE)
        cs.show_all()
        cs.help_button.hide()
        cs.ok_button.connect_object('clicked',self.color_changed,cs,key)
        cs.cancel_button.connect('clicked',lambda x,y:y.destroy(),cs)

    def color_changed(self,selector,key):
        import string 
        color_channel = selector.colorsel.get_color()        
        red, green, blue = color_channel[0],color_channel[1],color_channel[2]

        tmpstyle = None
        
        if key == 'font-color':
            tmpstyle = PrefModel().get_current_style().copy()
            for i in range(5):
                tmpstyle.fg[i] = selector.get_colormap().alloc(int(red*65535),
                                                               int(green*65535),
                                                               int(65535*blue))
        elif key == 'border-color':
            tmpstyle = PrefModel().get_current_style().copy()
            for i in range(5):
                tmpstyle.dark[i] = selector.get_colormap().alloc(int(red*65535),
                                                                 int(green*65535),
                                                                 int(65535*blue))
        elif key == 'background':
            tmpstyle = PrefModel().get_current_style().copy()
            for i in range(5):
                tmpstyle.bg[i] = selector.get_colormap().alloc(int(red*65535),
                                                                 int(green*65535),
                                                                 int(65535*blue))
        elif key == 'popped-color':
            PrefModel().set_popped_color(selector.get_colormap().alloc(int(red*65535),
                                                                             int(green*65535),
                                                                             int(65535*blue)))
        elif key == 'pushed-color':
            PrefModel().set_pushed_color(selector.get_colormap().alloc(int(red*65535),
                                                                             int(green*65535),
                                                                             int(blue*65535)))
        if tmpstyle:
            PrefModel().set_current_gtkstyle(tmpstyle)
            if self._type == 'step':
                PrefModel().update_sample_from_change(self._type,self._state_step)
            else:
                PrefModel().update_sample_from_change(self._type,self._state_transition)
                
        selector.destroy()

    # choose font callback #

    def choose_font(self,btn,key):
        import string 
        fs = GtkFontSelectionDialog(unicode_to_gtk(_('Choose label font')))
        fs.set_font_name(PrefModel().get_current_font())
        fs.set_position(WIN_POS_MOUSE)
        fs.show_all()
        fs.apply_button.hide()
        fs.ok_button.connect_object('clicked',self.font_changed,fs,key)
        fs.cancel_button.connect('clicked',lambda x,y:y.destroy(),fs)

    def font_changed(self,selector,key):
        PrefModel().set_current_gtkfont(selector.fontsel.get_font_name())
        if self._type == 'step':
            PrefModel().update_sample_from_change(self._type,self._state_step)
        else:
            PrefModel().update_sample_from_change(self._type,self._state_transition)
        selector.destroy()

    # destroy plan callback #

    def destroy_plan(self,*args):
        PrefModel().update_destroy(args[0].get_active())
