# Copyright (c) 2000-2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""Preferences Model"""

__revision__ = "$Id: PrefModel.py,v 1.7 2001/10/08 16:13:45 syt Exp $"

from Observer import Observable
from xml.xpath import Evaluate
from xml.dom.ext import Print,PrettyPrint

from narval.lib import *
from os.path import normpath,normcase,join
from string import strip,split

import sys

class PrefModel(Observable):
    """
    model of user preferences using singleton pattern
    """

    _DICT_ = {}

    def __init__(self):
        self.__dict__ = self._DICT_

        if not self.__dict__ :
            Observable.__init__(self)

            self._pref_tree = None
            # recipe style
            self._list_step_style = {}
            self._list_transition_style = {}
            self._list_step_font = {}
            self._list_transition_font = {}
            self._current_style = None
            self._current_font = ''
            # colour allocation
            self._popped_colour = None
            self._pushed_colour = None
            # destroy plan or not
            self._destroy_plan = 0

    ### PUBLIC ###

    # get pushed color
    def get_pushed_color(self):
        return self._pushed_colour

    # set pushed color
    def set_pushed_color(self,color):
        self._pushed_colour = color
        self.changes.append(("pushed_color",))
        self.notify_observers()
        
    # get popped color
    def get_popped_color(self):
        return self._popped_colour

    # set popped color
    def set_popped_color(self,color):
        self._popped_colour = color
        self.changes.append(("popped_color",))
        self.notify_observers()

    # update element on recipe
    def get_style_from_state(self,type,state):
        if type == 'step':
            return self._list_step_style[state]
        else:
            return self._list_transition_style[state]

    # recipe style functions
    def set_current_gtkstyle(self,style):
        self._current_style = style

    def set_current_gtkfont(self,fontname):
        self._current_font = fontname
        from gtk import load_font
        self._current_style.font = load_font(fontname)

    def get_current_style(self):
        return self._current_style

    def get_current_font(self):
        return self._current_font

    def get_list_step_style(self):
        return self._list_step_style

    def get_list_transition_style(self):
        return self._list_transition_style

    # preferences file functions

    def set_pref(self,filename):
        self._filename = filename
        cat_path = normcase(normpath(join(config.get_home(),'dtd/CATALOG')))
        self._pref_tree = read_domtree_from_file(filename,validate=1,catName=cat_path)
        self._build_style_list()

    def get_pref_tree(self):
        return self._pref_tree

    def get_pref_filename(self):
        return self._filename

    def save_preferences(self):
        file = open(self._filename,'w')
        head = '<?xml version="1.0" encoding="iso-8859-1" standalone="no" ?>\n' +\
               '<!DOCTYPE preferences PUBLIC  "-//Logilab//DTD Horn preferences//EN"' +\
               ' "horn_preferences.dtd" []>\n'
        
        file.write(head)

        self._build_tree_from_list()
        PrettyPrint(self._pref_tree.documentElement,file)
        file.close

    # update sample function

    def update_sample_from_menu(self,type,state):
        # just an update from model to view for the sample
        from PrefView import PrefView
        if type == 'step':
            self._current_style = self._list_step_style[state]
            self._current_font = self._list_step_font[state]
        elif type == 'transition':
            self._current_style = self._list_transition_style[state]
            self._current_font = self._list_transition_font[state]
            
        PrefView().update_sample(self._current_style)

    def update_sample_from_change(self,type,state):
        if type == 'step':
            self._list_step_style[state] = self._current_style
            self._list_step_font[state] = self._current_font
        elif type == 'transition':
            self._list_transition_style[state] = self._current_style
            self._list_transition_font[state] = self._current_font

        from PrefView import PrefView
        PrefView().update_sample(self._current_style)

    def update_destroy(self,*args):
        self._destroy_plan = args[-1]
        self.changes.append(('destroy',args[-1]))
        self.notify_observers()

    ### PRIVATE ###

    def _build_style_list(self):
        from gtk import *
        parts = select_many(self._pref_tree,'/preferences/part')
        node = select_one(parts[0],'entries')
        element = node.firstChild
        while element:
            state = element.firstChild.firstChild
            while state:
                style = state.firstChild
                while style:
                    tmp = GtkLabel("")
                    gtkstyle = tmp.get_style().copy()
                    
                    # load values
                    fontname = strip(select_one(style,'./entry[@name="font"]/value/text()').data)
                    font = load_font(fontname)
                    bg = split(select_one(style,'./entry[@name="background"]/value/text()').data)
                    font_color = split(select_one(style,'./entry[@name="font-color"]/value/text()').data)
                    border_color = split(select_one(style,'./entry[@name="border-color"]/value/text()').data)
                            
                    gtkstyle.font = font
                    for i in range(5):
                        gtkstyle.fg[i] = tmp.get_colormap().alloc(int(font_color[0]),int(font_color[1]),int(font_color[2]))
                        gtkstyle.bg[i] = tmp.get_colormap().alloc(int(bg[0]),int(bg[1]),int(bg[2]))
                        gtkstyle.dark[i] = tmp.get_colormap().alloc(int(border_color[0]),
                                                                    int(border_color[1]),
                                                                    int(border_color[2]))
                        gtkstyle.light[i] = tmp.get_colormap().alloc(int(border_color[0]),
                                                                     int(border_color[1]),
                                                                     int(border_color[2]))
                        gtkstyle.base[i] = tmp.get_colormap().alloc(int(border_color[0]),
                                                                    int(border_color[1]),
                                                                    int(border_color[2]))
                    if element.getAttributeNS('','name') == 'step':
                        self._list_step_style[state.getAttributeNS('','name')] = gtkstyle
                        self._list_step_font[state.getAttributeNS('','name')] = fontname
                    elif element.getAttributeNS('','name') == 'transition':
                        self._list_transition_style[state.getAttributeNS('','name')] = gtkstyle
                        self._list_transition_font[state.getAttributeNS('','name')] = fontname

                    style = style.nextSibling
                state = state.nextSibling
            element = element.nextSibling
        # color messages part
        popcol = split(select_one(parts[1],'./entry[@name="popped-color"]/value/text()').data)
        self._popped_colour = tmp.get_colormap().alloc(int(popcol[0]),int(popcol[1]),int(popcol[2]))
        pushcol = split(select_one(parts[1],'./entry[@name="pushed-color"]/value/text()').data)
        self._pushed_colour = tmp.get_colormap().alloc(int(pushcol[0]),int(pushcol[1]),int(pushcol[2]))

        # destroy plan part
        self._destroy_plan = int(select_one(parts[2],'./entry[@name="destroy"]/value/text()').data)

        self.changes.append((self._destroy_plan,self._current_style))
        self.notify_observers()

    # rebuild tree from new values in list

    def _build_tree_from_list(self):
        if sys.platform=='win32':
            from GTKconst import *
        else:
            from GTK import *
        parts = select_many(self._pref_tree,'/preferences/part')
        node = select_one(parts[0],'entries')
        element = node.firstChild
        while element:
            state = element.firstChild.firstChild
            while state:
                style = state.firstChild
                while style:
                    if element.getAttributeNS('','name') == 'step':
                        select_one(style,'./entry[@name="font"]/value/text()').data =\
                        self._list_step_font[state.getAttributeNS('','name')]
                        
                        select_one(style,'./entry[@name="background"]/value/text()').data = \
                        str(self._list_step_style[state.getAttributeNS('','name')].bg[STATE_NORMAL].red)+" "+\
                        str(self._list_step_style[state.getAttributeNS('','name')].bg[STATE_NORMAL].green)+" "+\
                        str(self._list_step_style[state.getAttributeNS('','name')].bg[STATE_NORMAL].blue)
                            
                        select_one(style,'./entry[@name="font-color"]/value/text()').data =\
                        str(self._list_step_style[state.getAttributeNS('','name')].fg[STATE_NORMAL].red)+" "+\
                        str(self._list_step_style[state.getAttributeNS('','name')].fg[STATE_NORMAL].green)+" "+\
                        str(self._list_step_style[state.getAttributeNS('','name')].fg[STATE_NORMAL].blue)
                            
                        select_one(style,'./entry[@name="border-color"]/value/text()').data =\
                        str(self._list_step_style[state.getAttributeNS('','name')].dark[STATE_NORMAL].red)+" "+\
                        str(self._list_step_style[state.getAttributeNS('','name')].dark[STATE_NORMAL].green)+" "+\
                        str(self._list_step_style[state.getAttributeNS('','name')].dark[STATE_NORMAL].blue)

                    elif element.getAttributeNS('','name') == 'transition':
                        select_one(style,'./entry[@name="font"]/value/text()').data =\
                        self._list_transition_font[state.getAttributeNS('','name')]
                            
                        select_one(style,'./entry[@name="background"]/value/text()').data =\
                        str(self._list_transition_style[state.getAttributeNS('','name')].bg[STATE_NORMAL].red)+" "+\
                        str(self._list_transition_style[state.getAttributeNS('','name')].bg[STATE_NORMAL].green)+" "+\
                        str(self._list_transition_style[state.getAttributeNS('','name')].bg[STATE_NORMAL].blue)
                            
                        select_one(style,'./entry[@name="font-color"]/value/text()').data =\
                        str(self._list_transition_style[state.getAttributeNS('','name')].fg[STATE_NORMAL].red)+" "+\
                        str(self._list_transition_style[state.getAttributeNS('','name')].fg[STATE_NORMAL].green)+" "+\
                        str(self._list_transition_style[state.getAttributeNS('','name')].fg[STATE_NORMAL].blue)
                            
                        select_one(style,'./entry[@name="border-color"]/value/text()').data =\
                        str(self._list_transition_style[state.getAttributeNS('','name')].dark[STATE_NORMAL].red)+" "+\
                        str(self._list_transition_style[state.getAttributeNS('','name')].dark[STATE_NORMAL].green)+" "+\
                        str(self._list_transition_style[state.getAttributeNS('','name')].dark[STATE_NORMAL].blue)


                    style = style.nextSibling
                state = state.nextSibling
            element = element.nextSibling
        select_one(parts[1],'./entry[@name="popped-color"]/value/text()').data=\
        str(self._popped_colour.red)+" "+str(self._popped_colour.green)+" "+str(self._popped_colour.blue)
        select_one(parts[1],'./entry[@name="pushed-color"]/value/text()').data=\
        str(self._pushed_colour.red)+" "+str(self._pushed_colour.green)+" "+str(self._pushed_colour.blue)

        from PrefView import PrefView
        select_one(parts[2],'./entry[@name="destroy"]/value/text()').data = str(PrefView().plan_button.get_active())
