# Copyright (c) 2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307
# USA.

"""
Recipe Model
"""

__revision__ = "$Id: RecipeModel.py,v 1.20 2001/10/08 10:18:28 syt Exp $"

from Observer import Observable
from narval.lib import *
from os.path import normpath,join
from xml.dom.ext import PrettyPrint
from xml.dom.ext.reader import Sax2
from RecipeLayout import *
from narval.gui import unicode_to_gtk

class RecipeModel(Observable):
    """
    Model of recipe which contains current modified recipe
    and unmodified recipe if user doesn't want to save
    """
    
    def __init__(self):
        Observable.__init__(self)
            
        # record the recipe which is going to be modified
        self._unmodified_recipe = None

        # current modified recipe
        self._recipe = None

        # record layout pattern
        self._layout_pattern = RecipeLayout1()

        # record pointer position
        self._pointer = (0,0) # FIXME qu'est-ce que a fait l !!!

    ### PUBLIC ###

    # pointer functions

    def set_pointer(self,pointer):
        self._pointer = pointer

    def get_pointer(self):
        return self._pointer

    # layout functions

    def get_layout_pattern(self):
        return self._layout_pattern

    def set_layout_pattern(self,lp):
        self._layout_pattern = lp
        self.changes.append(('pattern',))
        self.notify_observers()

    # get recipe functions
        
    def get_unmodified_recipe(self):
        return self._unmodified_recipe

    def get_recipe(self):
        return self._recipe

    def set_recipe(self,recipe):
        self._unmodified_recipe = Sax2.FromXml(element_to_string(recipe)).documentElement
        self._recipe = Sax2.FromXml(element_to_string(recipe)).documentElement
        # update graphic list before...
        self.new = 1
        self.changes.append(())
        self.notify_observers()

    def update_recipe(self,recipe):
        self._recipe = recipe
        self.new = 0
        self.changes.append(())
        self.notify_observers()
    
    # step functions ###########################################

    def add_step(self,id,node=None):
        if node:
            node.setAttributeNS(AL_NS,'id',id)
            self._recipe.appendChild(node)
        else:
            step = self._recipe.ownerDocument.createElementNS(AL_NS,'step')
            step.setAttributeNS(AL_NS,'target','No target')
            step.setAttributeNS(AL_NS,'id',id)
            self._recipe.appendChild(step)
        self.changes.append(())
        self.notify_observers()

    def remove_step(self,id):
        # remove connections before
        for inref in select_many(self._recipe,'transition/in[@idref="%s"]'%(id)):
            if inref:
                inref.parentNode.removeChild(inref)
                release_domtree(inref)
        for outref in select_many(self._recipe,'transition/out[@idref="%s"]'%(id)):
            if outref:
                outref.parentNode.removeChild(outref)
                release_domtree(outref)
        try:
            step = select_one(self._recipe,'step[@id="%s"]'%(id))
            step.parentNode.removeChild(step)
            release_domtree(step)
        except:
            import traceback
            traceback.print_exc()
            print unicode_to_gtk(_('step %s not found') % str(id))

        self.changes.append(())
        self.notify_observers()

    def update_step(self,id,newstep):
        # FIXME : dirty method...
        try:
            tmpstep = select_one(self._recipe,'step[@id="%s"]'%(id))
            new = tmpstep.parentNode.ownerDocument.importNode(newstep,DEEP_COPY)
            tmpstep.parentNode.replaceChild(new,tmpstep)
            self.changes.append(())
            self.notify_observers()
        except:
            import traceback
            traceback.print_exc()
            print unicode_to_gtk(_('step %s not found') % str(id))

    # transition function ##################################

    def add_transition(self,id,id_in=None,id_out=None):
        transition = self._recipe.ownerDocument.createElementNS(AL_NS,'transition')
        transition.setAttributeNS(AL_NS,'id',id)
        self._recipe.appendChild(transition)
        if id_in:
            link_in = transition.ownerDocument.createElementNS(AL_NS,'in')
            link_in.setAttributeNS(AL_NS,'idref',id_in)
            transition.appendChild(link_in)

        if id_out:
            link_out = transition.ownerDocument.createElementNS(AL_NS,'out')
            link_out.setAttributeNS(AL_NS,'idref',id_out)
            transition.appendChild(link_out)
            
        self.changes.append(())
        self.notify_observers()

    def add_in_link_to_transition(self,id,id_in):
        transition = select_one(self._recipe,'transition[@id="%s"]'%(id))
        link_in = transition.ownerDocument.createElementNS(AL_NS,'in')
        link_in.setAttributeNS(AL_NS,'idref',id_in)
        transition.appendChild(link_in)
        self.changes.append(())
        self.notify_observers()

    def add_out_link_to_transition(self,id,id_out):
        transition = select_one(self._recipe,'transition[@id="%s"]'%(id))
        link_out = transition.ownerDocument.createElementNS(AL_NS,'out')
        link_out.setAttributeNS(AL_NS,'idref',id_out)
        transition.appendChild(link_out)
        self.changes.append(())
        self.notify_observers()        
        
    def remove_transition(self,id):
        try:
            transition = select_one(self._recipe,'transition[@id="%s"]'%(id))
            transition.parentNode.removeChild(transition)
            release_domtree(transition)
        except:
            import traceback
            traceback.print_exc()

        self.changes.append(())
        self.notify_observers()

    def update_transition(self,id,newtransition):
        # FIXME : dirty method...
        try:
            tmptransition = select_one(self._recipe,'transition[@id="%s"]'%(id))
            new = tmptransition.parentNode.ownerDocument.importNode(newtransition,DEEP_COPY)
            tmptransition.parentNode.replaceChild(new,tmptransition)
            self.changes.append(())
            self.notify_observers()
        except:
            import traceback
            traceback.print_exc()
