# Copyright (c) 2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

__revision__ = '$Id: Classify.py,v 1.3 2001/10/10 08:05:34 syt Exp $'

from narval.lib import *
source_vector = load_extension_module('classify', 'source_vector')
process_vectors = load_extension_module('classify', 'process_vectors')
sort_vectors_mc = load_extension_module('classify', 'sort_vectors_mc')


MOD_XML = """<?xml version="1.0" encoding="ISO-8859-1"?>
<module name='Classify'>"""
    
# Sort with term freq (optional) and mobile point algorithm ###################
def sort_tf_mc_f(args) :
    """
    Algo:
        data list => long vector list (use_engl_dico=0, nb_mini_letters=3)
        long vector list => vector list
        vector list => sorted tree
        sorted tree => sort-group
    """
    doc = args.ownerDocument
    output = doc.createElementNS('', 'outputs')
    debug = 0   # to debug in Narval
    vector_list=[]
    list_error=[]
    s_u_l = select_many(args, 'data-unit')
    try:
        tf_idf = select_one(args, 'tf-idf/text()')
        if tf_idf == 'no':
            tf_idf = None
        elif not tf_idf == 'yes':
            raise Exception
    except:
        if len(s_u_l) < 50:
            tf_idf = None
        else:
            tf_idf = 1
    try:
        class_nb = int(select_one(args, 'classes-number/text()').data)
    except:
        class_nb = None
    for s_u in s_u_l:
        tmp_url_vect = []
        from cStringIO import StringIO
        data = StringIO(select_one(s_u, 'data/text()').data)
        # FIXME parameters use_engl_dico, nb_mini_letters, dico may be fixed
        tsv = source_vector.tranf_source_vector(use_engl_dico=1,
                                                nb_mini_letters=4)
        tmp_url_vect.append(s_u)
        try:
            tmp_url_vect.append(tsv.do(data, debug=debug, only_dico=0))
        except source_vector.ReadError:
            # Error, Pass this data.
            list_error.append(s_u)
            if debug:
                print 'error-sourcevector'
            continue
        vector_list.append(tmp_url_vect)
    
    # return an empty node if no news
    if len(vector_list) == 0:
        s_g = doc.createElementNS('', 'sort-group')
        output.appendChild(s_g)
        return output

    # search a classes number value or performs it
    if not class_nb or class_nb < 0:
        if len(vector_list) < 12:
            class_nb = 4
        else:
            class_nb = int(len(vector_list)/5)

    if tf_idf:
        # process tf-idf on vectors
        p_vect = process_vectors.process_vectors(vector_list, debug)
        vector_list = p_vect.do()
    
    # vector list => sorted tree
    svt = sort_vectors_mc.sort_vectors_mc(vector_list, class_nb, debug)
    the_tree = svt.do()

    # sorted tree => sort-group
    for classe in the_tree:
        # Search common words in all articles and process title
        txt_title = common_words(classe, vector_list, debug)
        s_g = doc.createElementNS('', 'sort-group')
        title = doc.createElementNS('', 'common')
        title.appendChild(doc.createTextNode(latin1_to_utf8(txt_title)))
        s_g.appendChild(title)
    
        for index in classe:
            s_g.appendChild(vector_list[index][0])
        output.appendChild(s_g)
        
    # insert error 
#    if len(list_error) != 0:
#        errors = doc.createElementNS('','sort-group')
#        title = doc.createElementNS('', 'common')
#        title.appendChild(doc.createTextNode('error'))
#        errors.appendChild(title)
#        for s_u in list_error[:]:
#            errors.appendChild(s_u)
 #       output.appendChild(errors)
    
    return output
    
MOD_XML = MOD_XML +  """
<action name='classify_tf_mc'
func='sort_tf_mc_f'>
<description>classify a list of data-unit elements in sort-group, with the mobil
point method and optionaly the tf-idf method. The tf-idf algorithm is used when
a tf-idf element is found or automatically when there are enough units to use
this method.</description>
<input use='yes' list='yes'>
<match>data-unit[data]</match>
<description lang='en'>sort unit, with a data child containing text for sort</description>
</input>
<input optional='yes'>
<description lang='en'>this element allow to specify the number of classes used by the
algorithms. If no classes number element is found, it will be adjusted according
to the number of units to sort.</description>
<match>classes-number</match>
</input>
<input optional='yes'>
<description lang='en'>the tf-idf method will be used or not if this node
contains 'yes' or 'no'. If this node isn't present, tf-idf will be automatically
used as soon as there are enough data to classify.
</description>
<match>tf-idf</match>
</input>
<output list='yes'>
<description lang='en'>groups containing data-unit childs and a common child with the
common words of the group</description>
<match>sort-group</match>
</output>
</action>
"""
MOD_XML = MOD_XML + "</module>"

def common_words(classe, vector_url_list, debug):
    """ search common words in all articles from classe
    return the first 5 words separated by ', '  
    """
    import types
    com_words = {}
    for index in classe:
        if type(index) == types.IntType:
            for duo in vector_url_list[index][1].items():
                if com_words.has_key(duo[0]):
                    com_words[duo[0]] = com_words[duo[0]] + duo[1]
                else:
                    com_words[duo[0]] = duo[1]
    # com_words => list and sort
    list_duo = com_words.items()
    misc = load_extension_module('classify', 'misc')
    list_duo.sort(misc.cmp_eq_words)
    txt_title = ''

    if debug:
        for i in range(10):
            if len(list_duo)-1 >= i:
                print list_duo[i]
            else:
                break
                
    for i in range(5):
        if len(list_duo)-1 >= i:
            txt_title = txt_title + str(list_duo[i][0]) + ', '
        else:
            break
    txt_title = txt_title[:-2]
    if len(txt_title) != 0:
        return txt_title
    else:
        return 'No words'
