# Copyright (c) 2001 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
from narval.lib import *
__revision__ = '$Id: Diff.py,v 1.2 2001/10/10 10:36:18 syt Exp $'

MOD_XML = """<module name='Diff'>"""

try:
    from logilab.xmldiff import input, fmes, format
except Exception:
    print 'Package logilab.xmldiff should be installed'
    raise Exception
    
def dom_remove_narval_attrs(node, uri):
    for attr_name in ['from_plan', 'from_step', 'timestamp', 'eid',
                      'input-id', 'output-id']:
        try:
            node.removeAttributeNS(uri, attr_name)
        except:
            pass
    
def tree_remove_narval_attrs(node):
    from logilab.xmldiff.objects import N_CHILDS, N_TYPE, NT_SYST, NT_ATTN
    index = 1
    for n in node[N_CHILDS][1:]:
        if n[N_TYPE] == objects.NT_SYST and \
           n[N_CHILDS][0][N_TYPE] == objects.NT_ATTN:
            if n.childs[0].name in ['from_plan', 'from_step', 'timestamp', 
                                    'eid', 'input-id', 'output-id']:
                node[N_CHILDS].pop(index)
                del n
            else: index += 1
        else:
            break
    
def xmldiff_f(args):
    """ extract differences between 2 dom trees """
    doc = args.ownerDocument
    output = doc.createElementNS('', 'outputs')
    ## retrieve arguments ##
    domtree1 = select_one(args, 'diffold').firstChild
    domtree2 = select_one(args, 'diffnew').firstChild
    try:
        rem_narval_attrs  = select_one(args, 'rem_narval_attrs').data
    except:
        rem_narval_attrs = 1
    ## remove Narval attributes ##
    if rem_narval_attrs:
        dom_remove_narval_attrs(domtree1, '')
        dom_remove_narval_attrs(domtree2, '')
    ## convert dom trees to internal tree ##
    tree1 = input.tree_from_dom(domtree1.firstChild())
    tree2 = input.tree_from_dom(domtree2.firstChild())

    ## apply tree to tree algorithm ##
    strategy = fmes.FmesCorrector(0.59, 0.5)
    actions = strategy.process_trees(tree1, tree2)

    ## format output ##
    xupd = format.xupdate_dom(doc, actions)
    
    output.appendChild(xupd)
    return output

MOD_XML = MOD_XML + \
          """
          <action name='xmldiff' func='xmldiff_f'>
            <input use='yes'><match>diffold</match></input>
            <input use='yes'><match>diffnew</match></input>
            <input optional='yes'><match>rem_narval_attrs</match></input>
            <output><match>modifications</match></output>
          </action> 
          """ 



def xmldiff_files_f(args):
    """ extract differences between 2 xml files """
    from xml.sax import SAXParseException
    import urllib
    doc = args.ownerDocument
    output = doc.createElementNS('', 'outputs')
    ## retrieve arguments ##
    url1 = select_one(args, 'oldurl/text()').data
    url2 = select_one(args, 'newurl/text()').data
    try:
        rem_narval_attrs = int(select_one(args, 'rem_narval_attrs').data)
    except:
        rem_narval_attrs = 1
    try:
        norm_sp = int(select_one(args, 'normalize').data)
    except:
        norm_sp = 1
    try:
        fh1 = urllib.urlopen(normalize_url(url1)[0])
        fh2 = urllib.urlopen(normalize_url(url2)[0])
    except IOError, msg :
        Err = doc.createElementNS('','error')
        Err.setAttributeNS('','type','No such url')
        output.appendChild(Err)
        return output

    ## convert xlm files to tree ##
    try:
        tree1 = input.tree_from_stream(fh1, norm_sp)
        tree2 = input.tree_from_stream(fh2, norm_sp)
        fh1.close (); fh2.close ()
    except SAXParseException, msg:
        Err = doc.createElementNS('','error')
        Err.setAttributeNS('','type','Error in xml file')
        output.appendChild(Err)
        return output
    
    # remove Narval attributes
    if rem_narval_attrs:
        tree_remove_narval_attrs(tree1)
        tree_remove_narval_attrs(tree2)
    ## apply tree to tree algorithm ##
    strategy = fmes.FmesCorrector(0.59, 0.5)
    actions = strategy.process_trees(tree1, tree2)
    ## format output ##
    xupd = format.xupdate_dom(doc, actions)
    
    output.appendChild(xupd)
    return output

MOD_XML = MOD_XML + \
          """
          <action name='xmldiff_files' func='xmldiff_files_f'>
            <input><match>oldurl/text()</match></input>
            <input><match>newurl/text()</match></input>
            <input optional='yes'><match>normalize</match></input>
            <input optional='yes'><match>rem_narval_attrs</match></input>
            <output><match>modifications</match></output>
          </action> 
          """ 



def xmldiff_file_elmt_f(args):
    """
    extract differences between a XLM file (old document) and a DOM tree (new document)
    """
    from xml.sax import SAXParseException
    import urllib
    doc = args.ownerDocument
    output = doc.createElementNS('', 'outputs')
    ## retrieve arguments ##
    url1 = select_one(args, 'oldurl/text()').data
    domtree2 = select_one(args, 'diffnew').firstChild
    try:
        rem_narval_attrs = int(select_one(args, 'rem_narval_attrs').data)
    except:
        rem_narval_attrs = 1
    try:
        norm_sp = int(select_one(args, 'normalize').data)
    except: 
        norm_sp = 1
    ## get norm tree from url1 ##
    try:
        fh1 = urllib.urlopen(normalize_url(url1)[0])
    except IOError, msg :
        Err = doc.createElementNS('','error')
        Err.setAttributeNS('','type','No such url')
        output.appendChild(Err)
        return output
    # convert xlm file to tree
    try:
        tree1 = input.tree_from_stream(fh1, norm_sp)
        fh1.close ()
    except SAXParseException, msg:
        Err = doc.createElementNS('','error')
        Err.setAttributeNS('','type','Invalide xml file')
        output.appendChild(Err)
        return output
    ## get norm tree from domtree2 ##
    # remove Narval attributes
    if rem_narval_attrs:
        tree_remove_narval_attrs(tree1)
        dom_remove_narval_attrs(domtree2, '')
    # convert dom trees to internal tree
    tree2 = input.tree_from_dom(domtree2)

    ## apply tree to tree algorithm ##
    strategy = fmes.FmesCorrector(0.59, 0.5)
    actions = strategy.process_trees(tree1, tree2)
    ## format output ##
    xupd = format.xupdate_dom(doc, actions)
    output.appendChild(xupd)
    return output

MOD_XML = MOD_XML + \
          """
          <action name='xmldiff_file_elmt' func='xmldiff_file_elmt_f'>
            <input><match>oldurl/text()</match></input>
            <input use='yes'><match>diffnew</match></input>
            <input optional='yes'><match>normalize</match></input>
            <input optional='yes'><match>rem_narval_attrs</match></input>
            <output><match>modifications</match></output>
          </action> 
          """ 

MOD_XML = MOD_XML + "</module>"
