# Copyright (c) 2000 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# to create a new download action, just add its type to the mime2al_type
# dictionnary

__version__ = "@version Tue Jul 18 11:20:40 2000 @@"

from narval.lib import *
import urlparse
from xml.dom.ext.reader import Sax2

MOD_XML='''<?xml version="1.0" encoding="ISO-8859-1"?><module name="Download">'''

##
## str2flat_tree and str2tree : these two functions convert the downloaded data
## into a tree with a type given as argument
## then, add its prototype to the MOD_XML string
## at last, add code to the guess_type function so that your new type is
## recognized completely, and add it to mime2al_type and doctype2al_type
##

def iso8859_to_utf8(data) :
    from xml.unicode.iso8859 import wstring
    import string
    wstring.install_alias('ISO-8859-1', 'ISO_8859-1:1987')
    s = wstring.decode('ISO-8859-1',data).utf8()
    return string.replace(s,'&#','  ') # remove remaining weirdnesses FIXME

def str2flat_tree(doc,string,a_type):
    ret=doc.createElementNS('',a_type)
    string = iso8859_to_utf8(string)
    ret.appendChild(doc.createCDATASection(string))
    return ret

def str2tree(doc,string,a_type):
    ret = doc.createElementNS('',a_type)
    string = iso8859_to_utf8(string)
    ret.appendChild(doc.importNode(Sax2.FromXml(string).documentElement,1))
    return ret


# al_type: what conversion to apply to the downloaded thing

al_type={'html':str2flat_tree,
         'rss':str2tree,
         'text':str2flat_tree,
         'xml':str2tree
         }


# mime2al_type : a dictionnary of al's types and function to read them indexed 
# by their mime type

mime2al_type={'text/html':'html',
              'text/rss':'rss',
              'text/plain':'text',
              'text/xml':'xml'}

# doctype2al_type : a dictionnary of al's types and function to read them
# indexed by their DOCTYPE type

doctype2al_type={'HTML':'html',
                 'rss':'rss'}

##
## guess_type : a function to guess the type of a downloaded file
##

class UnknownType(Exception):
    pass


def try_url(url,url_tuple,data):
    if url[-4:] in ['php3','php4','html']:
        return 'html'
    if url[-3:]=='htm':
        return 'html'
    if url[-4:] == '.rdf' :
        return 'rss'
    return None

def try_doctype(url,url_tuple,data):
    #if data[:5] != '<?xml' : return
    from re import compile
    m = re.compile(r"<!DOCTYPE\s+([a-zA-Z]+)\s+", re.DOTALL)
    result = m.search(data)
    if result:
        return result.group(1)
    return None

def guess_al_type(url,url_tuple,data):
    for try_func in [try_url,try_doctype]:
        guessed_type = try_func(url,url_tuple,data)
        if guessed_type:
            try:
                return doctype2al_type[guessed_type]
            except KeyError:
                pass
    raise UnknownType

##
## dl_* functions : functions to download something, depending on the protocol
##

def dl_http(doc,args,url,url_tuple):
    from httplib import HTTP
    try:
        proxy=select_one(args,'proxy[@type="http"]')
        request=HTTP(proxy.getAttributeNS('','hostname')+':'+\
                     proxy.getAttributeNS('','port'))
        request.putrequest('GET',url)
    except SelectException:
        server_port=url_tuple[1]
        i=len(server_port)
        j=i
        port=80
        while i>0:
            i=i-1
            if server_port[i]==':':
                try:
                    port=int(server_port[i+1:])
                except ValueError:
                    ret=doc.createElementNS('','error')
                    ret.appendChild(doc.createTextNode('bad url format'))
                    return ret
                j=i
        request=HTTP(server_port[:j],port)
        request.putrequest('GET',urlparse.urlunparse(('','',url_tuple[2],
                                                     url_tuple[3],url_tuple[4],
                                                     url_tuple[5])))
    request.endheaders()
    replycode,message,headers=request.getreply()
    if replycode==200:
         file=request.getfile()
         data=file.read()
         file.close()
    else:
        ret=doc.createElementNS('','error')
        ret.appendChild(doc.createTextNode('%i %s'%(replycode,message)))
        return ret

    # find out document type -- try to be smarter than lazily configured
    # web servers.
    try:
        mime_type = mime2al_type[headers.gettype()]
    except KeyError:
        print "Unknown MIME type %s" % headers.gettype()
        mime_type = None

    try:
        guessed_type = guess_al_type(url,url_tuple,data)
    except UnknownType :
        guessed_type = None
        
    if guessed_type and guessed_type != mime_type :
        print "Guessed type is %s and MIME type is %s" % (guessed_type, mime_type)
        return al_type[guessed_type](doc,data,guessed_type)
    elif mime_type :
        return al_type[mime_type](doc,data,mime_type)
    else :
        return str2flat_tree(doc,data,'text')


        
def dl_urllib(doc,args,url,url_tuple):
    proxies={}
    for proxy in select_many(args,'proxy'):
        proxies[proxy.getAttributeNS('','type')]=\
            proxy.getAttributeNS('','type')+'://'+\
            proxy.getAttributeNS('','hostname')+':'+\
            proxy.getAttributeNS('','port')
    import urllib
    try:
        z=urllib.URLopener(proxies).open(url)
        data=z.read()
        z.close()
    except IOError,e:
        ret=doc.createElementNS('','error')
        ret.appendChild(doc.createTextNode('an error occured during download:%s')%str(e))
        try:
            z.close()
        except:
            pass
        return ret
    t = guess_al_type(url,url_tuple,data)
    return al_type[t](doc,data,t)


##
## download
##

def Download(args):
    url=select_one(args,'url/text()').data
    url_tuple=urlparse.urlparse(url)
    doc=args.ownerDocument
    ret=doc.createElementNS('','outputs')

    if url_tuple[0] == 'http' :
        ret.appendChild(dl_http(doc,args,url,url_tuple))
    elif url_tuple[0] in ['ftp','gopher','file'] :
        ret.appendChild(dl_urllib(doc,args,url,url_tuple))
    else :
        err=doc.createElementNS('','error')
        err.appendChild(doc.createTextNode('unknown protocol'))
        ret.appendChild(err)
    return StripXml(ret)

MOD_XML=MOD_XML+'''
<action name="Download_rss" func="Download">
  <description lang="en">Download a RSS ressource (Netscape RDF format)
  </description>
  <description lang="fr">Tlcharge une ressource RSS (format RDF de Netscape)
  </description>
  <input><match>url/text()</match></input>
  <input optional='yes' list='yes'><match>proxy</match></input>
  <output><match>rss</match></output>
</action>
<action name="Download_html" func="Download">
  <description lang="en">Download a HTML document</description>
  <description lang="fr">Tlcharge un document HTML</description>
  <input><match>url/text()</match></input>
  <input optional='yes' list='yes'><match>proxy</match></input>
  <output><match>html/text()</match></output>
</action>
'''

MOD_XML=MOD_XML+'</module>'

if __name__ == "__main__" :
    print MOD_XML
    doc=Sax2.FromXml(MOD_XML)
    print doc.documentElement
