# Copyright (c) 2000 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

__revision__ = "$Id: Email.py,v 1.19 2001/08/14 12:46:22 adim Exp $"

"""
Narval Email module. Refer to dtd/Email.dtd for e-mail structure.
"""

from narval.lib import *
from narval import config

import smtplib
import string
import multifile, mimetools, mimetypes, mimify
import mailbox
from StringIO import StringIO

MAIL_NS = ''

MOD_XML = '''<?xml version="1.0" encoding="ISO-8859-1"?><module name="Email">'''

# Send an e-mail ###############################################################

def send_f(args) :
    """Take an <email> as input and send it"""

    import MimeWriter

    # mail headers
    doc = args.ownerDocument
    headers = select_one(args,'email/headers')
    frm = "%s <%s>" % (utf8_to_latin1(select_one(headers,'from/name/text()').data),
                       utf8_to_latin1(select_one(headers,'from/email_addr/text()').data))
    to = utf8_to_latin1(select_one(headers,'to/text()').data)
    try:
        cc = utf8_to_latin1(select_one(headers,'cc/text()').data)
    except SelectException :
        cc = ""
    try:
        subject = utf8_to_latin1(select_one(headers,'subject/text()').data)
    except SelectException :
        subject = "No subject"
        
    # mail body
    body = select_one(args,'email/body')
    if not body.getAttributeNS(MAIL_NS,'multipart') :
        if body.firstChild:
            text = '\n'+ utf8_to_latin1(body.firstChild.data)
        else:
            text = ''
    else:
        # multipart message
        parts = select_many(body,'part')
        buffer = StringIO()
        multipart_doc = MimeWriter.MimeWriter(buffer)
        #multi_part_doc.addheader('Content-Type','Multipart/Mixed')
        multipart_doc.startmultipartbody('Mixed')
       
        # write down parts
        for part in parts:
            subwriter = multipart_doc.nextpart()
            subwriter.addheader('Content-Transfert-Encoding',
                                part.getAttributeNS(MAIL_NS,'encoding'))
            if  part == parts[0]:
                subody = subwriter.startbody(part.getAttributeNS(MAIL_NS,
                                                            'content_type'))
            else:
                file_name = str(string.split(part.getAttributeNS(MAIL_NS,
                                                                 'name'),
                                             '/'))
                subwriter.addheader('Content-Disposition',
                                    'attachement; filename = "%s"'%(file_name))
                subody = subwriter.startbody(part.getAttributeNS\
                                             ('',
                                              'content_type'))
            subody.write(part.firstChild.data)
        multipart_doc.lastpart()
        text = buffer.getvalue()

    # signature
    try:
        sign = select_one(args,'email/signature/text()').data
    except SelectException:
        sign = '\n-- \nNarval your pal'
    envelope_dest = string.split(to,', ')
    if cc != '':
        cc_list = string.split(cc,', ')
        map(lambda x,l=envelope_dest: l.append(x), cc_list)

    msg = 'From: %s\r\nTo: %s\r\nCc: %s\r\nSubject: %s\r\n' % \
          (str(frm),str(to),str(cc),str(subject))
    msg = msg + '%s\r\n\r\n%s' % (str(text),str(sign))
    # send mail
    send = smtplib.SMTP('mail')
    #send.set_debuglevel(1)
    send.sendmail(str(frm),envelope_dest,msg)
    send.quit()
    outputs = doc.createElementNS('','outputs')
    return outputs


MOD_XML = MOD_XML + \
          '''<action name="send" func="send_f">
          <input use="yes"><match>email</match></input>
          </action>
          '''

# RFC822 TO XML ###############################################################

def rfc822_to_xml(msg,doc) :

    r = doc.createElementNS(MAIL_NS,'email')
    headers = doc.createElementNS(MAIL_NS,'headers')
    r.appendChild(headers)
    r.setAttributeNS(MAIL_NS,'message-id',msg.get('message-id'))
    frm = msg.getaddr('from')
    frmNode = doc.createElementNS(MAIL_NS,'from')
    name = doc.createElementNS(MAIL_NS,'name')
    name.appendChild(doc.createTextNode(latin1_to_utf8(frm[0])))
    frmNode.appendChild(name)
    email_addr = doc.createElementNS(MAIL_NS,'email_addr')
    email_addr.appendChild(doc.createTextNode(latin1_to_utf8(frm[1])))
    frmNode.appendChild(email_addr)
    headers.appendChild(frmNode)

    if msg.get('to') :
        to = doc.createElementNS(MAIL_NS,'to')
        to.appendChild(doc.createTextNode(latin1_to_utf8(msg.get('to'))))
        headers.appendChild(to)

    if msg.get('cc') :
        cc = doc.createElementNS(MAIL_NS,'cc')
        cc.appendChild(doc.createTextNode(latin1_to_utf8(msg.get('cc'))))
        headers.appendChild(cc)

    if msg.get('subject') :
        subject = doc.createElementNS(MAIL_NS,'subject')
        subject.appendChild(doc.createTextNode(latin1_to_utf8(msg.get('subject'))))
        headers.appendChild(subject)

    if msg.get('date') :
        date = doc.createElementNS(MAIL_NS,'date')
        from time import strftime
        from rfc822 import parsedate
        datestr = strftime ('%c', parsedate (msg.get ('date')))
        date.appendChild (doc.createTextNode (latin1_to_utf8(datestr)))
        headers.appendChild(date)
    
    if msg.get('content-type') :
        subcontenttype = doc.createElementNS(MAIL_NS,'content-type')
        subcontenttype.appendChild(doc.createTextNode(latin1_to_utf8(msg.get('content-type'))))
        headers.appendChild(subcontenttype)

    if msg.get('text') :
        headers = doc.createElementNS(MAIL_NS,'text')
        headers.appendChild(doc.createTextNode(latin1_to_utf8(msg.get('headers'))))
        headers.appendChild(text)

    b = doc.createElementNS(MAIL_NS,'body')
    r.appendChild(b)

    if msg.getmaintype() == 'multipart':
        b.setAttributeNS(MAIL_NS,'multipart','yes')
        read_multipart(msg,b,doc)
    else:
        b.setAttributeNS(MAIL_NS,'multipart','no')
##    if msg.getmaintype() == 'multipart' :
##        b.setAttributeNS(MAIL_NS,'multipart','yes')
##        msg.rewindbody()
##        read_multipart(msg,b,doc)
##    else :
##        b.appendChild(doc.createTextNode(latin1_to_utf8( msg.fp.read())) )
    return r


def read_multipart(m,b,doc) :
    # look for boundary
    m.rewindbody()
    mf = multifile.MultiFile(m.fp)
    mf.push(m.getparam('BOUNDARY'))
    mf.next()
    # read first part (supposedly text)
    msg_text = mimetools.Message(StringIO(mf.read()))
    msg_text.rewindbody()
    part = doc.createElementNS(MAIL_NS,'part')
    part.setAttributeNS(MAIL_NS,'type','text')
    part.setAttributeNS(MAIL_NS,'content_type','text/plain')
    part.setAttributeNS(MAIL_NS,'encoding','8bit')
    
    b.appendChild(part)
   
    # read other parts, *** for now discard subparts ***
    while mf.next() :
        part = mimetools.Message(StringIO(mf.read()))
        encoding = part.getencoding()
        part.rewindbody()
        print 'WARNING:',m.get('message-id')
        if part.getparam('name') :
            fichier = part.getparam('name')
        else :
            fichier = m.get('message-id')
        if encoding == '7bit' or encoding == '8bit' :
            #open(fichier,'w').write(part.fp.read())
            data = part.fp.read()
        else :
            #mimetools.decode(part.fp,open(fichier,'w'),encoding)
            data_f = StringIO()
            mimetools.decode(part.fp,data_f,encoding)
            # data = data_f.getvalue()
        p = doc.createElementNS(MAIL_NS,'part')
        p.setAttributeNS(MAIL_NS,'type','file')
        p.setAttributeNS(MAIL_NS,'content_type', part.get('content_type'))
        p.setAttributeNS(MAIL_NS,'encoding',encoding)
        p.setAttributeNS(MAIL_NS,'name',fichier)
        # p.appendChild(doc.createTextNode(latin1_to_utf8(data)))
        b.appendChild(p) 


##def read_multipart(m,b,doc) :
##    # look for boundary
##    m.rewindbody()
##    mf = multifile.MultiFile(m.fp)
##    mf.push(m.getparam('BOUNDARY'))
##    mf.next()
##    # read first part (supposedly text)
##    msg_text = mimetools.Message(StringIO(mf.read()))
##    msg_text.rewindbody()
##    part = doc.createElementNS(MAIL_NS,'part')
##    part.setAttributeNS(MAIL_NS,'type','text')
##    part.setAttributeNS(MAIL_NS,'content_type','text/plain')
##    part.setAttributeNS(MAIL_NS,'encoding','8bit')

##    part.appendChild(doc.createTextNode(latin1_to_utf8(msg_text.fp.read())))
##    b.appendChild(part)
   
##    # read other parts, *** for now discard subparts ***
##    while mf.next() :
##        part = mimetools.Message(StringIO(mf.read()))
##        encoding = part.getencoding()
##        part.rewindbody()
##        print 'WARNING:',m.get('message-id')
##        if part.getparam('name') :
##            fichier = part.getparam('name')
##        else :
##            fichier = m.get('message-id')
##        if encoding == '7bit' or encoding == '8bit' :
##            #open(fichier,'w').write(part.fp.read())
##            data = part.fp.read()
##        else :
##            #mimetools.decode(part.fp,open(fichier,'w'),encoding)
##            data_f = StringIO()
##            mimetools.decode(part.fp,data_f,encoding)
##            data = data_f.getvalue()
##        p = doc.createElementNS(MAIL_NS,'part')
##        p.setAttributeNS(MAIL_NS,'type','file')
##        p.setAttributeNS(MAIL_NS,'content_type', part.get('content_type'))
##        p.setAttributeNS(MAIL_NS,'encoding',encoding)
##        p.setAttributeNS(MAIL_NS,'name',fichier)
##        p.appendChild(doc.createTextNode(latin1_to_utf8(data)))
   
##        b.appendChild(p) 

class UnixMailbox (mailbox.UnixMailbox) :
    def next(self):
        """fonction qui renvoie un couple compos de la 1ere ligne du document
        reprsentant le mail ainsi qu'un objet de type mimetools.Message
        contenant toutes les autres informations
        """ 
        while 1:
            self.fp.seek(self.seekp)
            try:
                self._search_start()
            except EOFError:
                self.seekp = self.fp.tell()
                return None
            start = self.fp.tell()
            self._search_end()
            self.seekp = stop = self.fp.tell()
            if start <> stop:
                break
        return mimetools.Message(mailbox._Subfile(self.fp, start, stop))


    def _isrealfromline(self, line):
        """For explanations, see
        home.netscape.com/eng/mozilla/2.0/relnotes/demo/content-length.html
        """
        return 1
    

# read new mail ################################################################

def read_new_f(args):
    """Read new mails and add an X-Narval mark in headers.
    """

    import time
    import os
    import stat

    doc = args.ownerDocument
    outputs = doc.createElementNS('','outputs')

    mailbox_def = select_one(args,'mailbox')
    last_check = mailbox_def.getAttributeNS('','mdate')
    if last_check != '':
        last_modif = os.stat(mailbox_def.getAttributeNS('','path'))[8]
        last_check = int(last_check)
        if last_check >= last_modif:
            new_mbox = doc.createElementNS('','mailbox')
            new_mbox.setAttributeNS('','path',mailbox_def.getAttributeNS('','path'))
            new_mbox.setAttributeNS('','mdate',mailbox_def.getAttributeNS('','mdate'))
            new_mbox.setAttributeNS('','persist','yes')

            outputs.appendChild(new_mbox)
            return outputs
        
    file = open(mailbox_def.getAttributeNS('','path'))
    
    # prend le verrou -- TODO FIXME
    # while !verrou_existe :
    #     sleep 1
    # open(mailbox_def[ATTR]['path']+".lock",'w')
    mbox = UnixMailbox(file)

    list_to_write = []

    # ajoute tous les messages  liste_to_write
    msg = mbox.next()
    while msg :
        # skip internal folder data
        discard_subject = "DON'T DELETE THIS MESSAGE -- FOLDER INTERNAL DATA"
        if msg and msg.get('Subject') == discard_subject :
            msg = mbox.next()
            continue
        
        # ajoute dlimiteur
        list_to_write.append('From narval %s\n' \
                             % time.asctime(time.localtime(time.time())))
        list_to_write = list_to_write + msg.headers

        # message dj visit
        if msg.get('X-Narval') :           
            print 'old message'
            list_to_write.append('\n')

        # pour les messages que Narval n'a pas visit    
        else:
            frm = msg.get('From')
            sub = msg.get('Subject')
            print '**** You have a new mail from', frm
            print '**** the subject is',sub
            list_to_write.append('X-Narval: I read that one :-)\n')
            list_to_write.append('\n')
            m = rfc822_to_xml(msg,doc)
            m.setAttributeNS(MAIL_NS,'type','incoming')
            outputs.appendChild(m)
            
        # Ecriture du corps du message
        msg.rewindbody()
        list_to_write = list_to_write + msg.fp.readlines()

        # au suivant !
        msg = mbox.next()

    # ferme bote aux lettres
    file.close()

    # On remplace l'ancien document contenant les mails par un nouveau o tous
    # les messages sont marqus.
    if list_to_write:
        open(mailbox_def.getAttributeNS('','path'),'w').writelines(list_to_write)

    mailbox_def.setAttributeNS('','mdate',str(os.stat(mailbox_def.getAttributeNS('','path'))[8]))
    mailbox_def.setAttributeNS('','persist','yes')
    outputs.appendChild(mailbox_def)
    # libre verrou
    # os.system("rm -f "+mailbox_def[ATTR]['path']+".lock")
    return outputs

MOD_XML = MOD_XML +\
          """<action name='read_new' func='read_new_f'>
          <input id='mail-box'><match>mailbox</match></input>
          <output list='yes' optional='yes'><match>email</match></output>
          <output outdates='mail-box'><match>mailbox</match></output>
          </action>"""

  

# Attached_file ################################################################

def save_attachment_f(args) :
    """fonction cherche les fichiers attachs. Pour l'insatnt elle ne
    fonctionne qu'avec un seul mail"""
    
    doc = args.ownerDocument
    outputs = doc.createElementNS('','outputs')
    
    email = select_one(args,'email')
    try:
        part_to_save = select_one(email,select_one(args,'part-xpath/text()').data)
    except SelectException:
        error = doc.createElementNS('','error')
        error.setAttributeNS('','attached-files','0')
        outputs.appendChild(error)
        return outputs
    
    mailbox = select_one(args,'mailbox')
    
    f = open(mailbox.getAttributeNS('','path'),'r')
    mbox = UnixMailbox(f)
    msg = mbox.next()
    while msg :
        # skip internal folder data
        discard_subject = "DON'T DELETE THIS MESSAGE -- FOLDER INTERNAL DATA"
        if msg and (msg.get('Subject') == discard_subject or msg.get('message-id') != email.getAttributeNS('','message-id')):
            msg = mbox.next()
            continue
        msg.rewindbody()
        mf = multifile.MultiFile(msg.fp)
        mf.push(msg.getparam('BOUNDARY'))
        mf.next()
        # read first part (supposedly text)
        msg_text = mimetools.Message(StringIO(mf.read()))
        msg_text.rewindbody()
        filename = part_to_save.getAttributeNS('','name')
        while mf.next() :
            part = mimetools.Message(StringIO(mf.read()))
            encoding = part.getencoding()
            part.rewindbody()
            if encoding == '7bit' or encoding == '8bit' :
                # open(fichier,'w').write(part.fp.read())
                data = part.fp.read()
            else :
                # mimetools.decode(part.fp,open(fichier,'w'),encoding)
                data_f = StringIO()
                mimetools.decode(part.fp,data_f,encoding)
                data = data_f.getvalue()
            import os
            save_path_element = select_one(args,'save-path')
            suffix = filename + save_path_element.getAttributeNS('','suffix')
            save_path = expand_vars(select_one(save_path_element,'text()').data)
            if os.path.isdir(save_path):
                import tempfile
                save_path = save_path + '/' + string.split(tempfile.mktemp(suffix),'/')[-1]

            saved_file = open(save_path,'w')    
            saved_file.write(data)
            saved_file.close()
            break
        
        break
    f.close()
        
    return outputs

MOD_XML = MOD_XML +\
          """
          <action name='save_attachment' func='save_attachment_f'>
          <input><match>mailbox</match></input>
          <input><match>save-path</match></input>
          <input use='yes'><match>part-xpath</match></input>
          <input use='yes'><match>email</match></input>
          <output optional='yes'><match>error</match></output>
          </action>
          """

def save_attachments_f(args) :
    """fonction cherche les fichiers attachs. Pour l'insatnt elle ne
    fonctionne qu'avec un seul mail"""
    
    doc = args.ownerDocument
    outputs = doc.createElementNS('','outputs')
    
    email = select_one(args,'email')
    attached_files = select_many(email,'body/part[@type="file"]')
    
    for file in attached_files:
        # browse all attached files 
        mailbox = select_one(args,'mailbox')
        file=attached_files[0]
        f = open(mailbox.getAttributeNS('','path'),'r')
        mbox = UnixMailbox(f)
        msg = mbox.next()
        while msg :
            # skip internal folder data
            discard_subject = "DON'T DELETE THIS MESSAGE -- FOLDER INTERNAL DATA"
            if msg and (msg.get('Subject') == discard_subject or msg.get('message-id') != email.getAttributeNS('','message-id')):
                msg = mbox.next()
                continue
            msg.rewindbody()
            mf = multifile.MultiFile(msg.fp)
            mf.push(msg.getparam('BOUNDARY'))
            mf.next()
            # read first part (supposedly text)
            msg_text = mimetools.Message(StringIO(mf.read()))
            msg_text.rewindbody()
            filename = file.getAttributeNS('','name')
            while mf.next() :
                part = mimetools.Message(StringIO(mf.read()))
                encoding = part.getencoding()
                part.rewindbody()
                # if part.getparam('name') != filename :
                #    continue
                if encoding == '7bit' or encoding == '8bit' :
                    # open(fichier,'w').write(part.fp.read())
                    data = part.fp.read()
                else :
                    # mimetools.decode(part.fp,open(fichier,'w'),encoding)
                    data_f = StringIO()
                    mimetools.decode(part.fp,data_f,encoding)
                    data = data_f.getvalue()
                import os
                save_path_element = select_one(args,'save-path')
                suffix = filename + save_path_element.getAttributeNS('','suffix')
                save_path = expand_vars(select_one(save_path_element,'text()').data)
                if os.path.isdir(save_path):
                    import tempfile
                    save_path = save_path + '/' + string.split(tempfile.mktemp(suffix),'/')[-1]

                saved_file = open(save_path,'w')    
                saved_file.write(data)
                saved_file.close()
                break

            break
    f.close()
        
    return outputs

MOD_XML = MOD_XML +\
          """
          <action name='save_attachments' func='save_attachments_f'>
          <input><match>mailbox</match></input>
          <input id='path'><match>save-path</match></input>
          <input use='yes'><match>email</match></input>
          </action>
          """



##
##  Attach file to mail
##

def attach_file_f(args):
    """A partir d'un document et d'un mail fait un mail multipart contenant le
    document
    """
    print "#########YOUHOU JE MARCHE"
    # get path
    doc=args.ownerDocument
    path = select_one(args,'document/path/text()').data
    doc_name = string.split(path,'/')[-1]

    # encode file
    data_f = StringIO()
    mimetools.encode(open(path),data_f,'base64')
    data = data_f.getvalue()
    # get email
    mail = select_one(args,'email')
    mail.setAttributeNS(MAIL_NS,'content_type','Multipart/Mixed')

    # get body
    body = select_one(mail,'body')
    # cas d'un mail multipart
    if body.getAttributeNS(MAIL_NS,'multipart')=="yes":
        pass
    # cas d'un mail non multipart
    else:
        try:
            text = select_one(mail,'body/text()').data
            body.removeChild(select_one(mail,'body/text()'))
        except SelectException:
            text = ''
        body.setAttributeNS(MAIL_NS,'multipart','yes')
        part1 = doc.createElementNS(MAIL_NS,'part')
        part1.setAttributeNS(MAIL_NS,'content_type','text/plain')
        part1.setAttributeNS(MAIL_NS,'type','text')
        part1.setAttributeNS(MAIL_NS,'encoding','8bit')
        part1.appendChild(doc.createTextNode(text))
        body.appendChild(part1)
        
    part2 = doc.createElementNS(MAIL_NS,'part')
    part2.setAttributeNS(MAIL_NS,'content_type',
                         mimetypes.guess_type(doc_name)[0])
    part2.setAttributeNS(MAIL_NS,'type','file')
    part2.setAttributeNS(MAIL_NS,'encoding','base64')
    part2.setAttributeNS(MAIL_NS,'name',path)
    part2.appendChild(doc.createTextNode(data))
    
    body = select_one(mail,'body')
    body.appendChild(part2)
    
    outputs = doc.createElementNS('','outputs')
    outputs.appendChild(mail)
    print "#########OUTPUTS"
    write_domtree(outputs)
    
    return outputs

MOD_XML = MOD_XML +\
          """<action name='attach_file' func='attach_file_f'>
          <input use='yes'><match>document</match></input>
          <input use='yes'><match>email</match></input>
          <output><match>email</match></output>
          </action>"""


##
##  Find an address in the mail
##

def find_doc_addr_f(args):
    """Cherche une addresse de fichier dans le sujet ou le corps du message
    et renvoie ce document  l'envoyeur
    """

    doc = args.ownerDocument
    subject = select_one(args,'email/headers/subject/text()').data
    list_of_words = string.split(subject,' ')
    #list = filter(lambda f:re.compile("file\s").match(f),list_of_words)
    i = 0
    list = []
    # Pour l'instant on demande  l'envoyeur d'crire
    # file Nom du fichier demand
    for word in list_of_words:
        i = i + 1
        if word == 'file':
            list.append (list_of_words[i])
    outputs = doc.createElementNS('','outputs')
    doc.documentElement.appendChild(outputs)
    document = doc.createElementNS('','document')
    outputs.appendChild(document)
    if list:
        doc_addr = list[0]
        
        path = doc.createElementNS('','path')
        path.appendChild(doc.createTextNode(doc_addr))
        document.appendChild(path)
        return outputs
    else:
        return outputs

MOD_XML = MOD_XML +\
          """<action name='find_doc_addr' func='find_doc_addr_f'>
          <input use='yes'><match>email</match></input>
          <output><match>document</match></output>
          </action>"""
    
MOD_XML = MOD_XML + "</module>"

if __name__ == "__main__" :
    print MOD_XML
    from xml.dom.ext.reader import Sax2
    doc=read_domtree_from_string(MOD_XML)
    print doc.documentElement
