# Copyright (c) 2000 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
__revision__ = '$Id: Http.py,v 1.34 2001/10/15 09:48:17 syt Exp $'
from narval.lib import *

import httplib
import urlparse
import htmllib
import formatter
import copy
import re
import os

try:
    lg_factory = load_extension_module('language','guess_language')
except:
    lg_factory = None

# set this variable to :
    # 0  for no junkbuster or external junkbuster
    # 1  for junkbuster based on server name
    # 2  for junkbuster based on server domain name 
    # 4  for junkbuster based on well-known patterns 
    # 8  for data filtering junkbuster (EXPERIMENTAL !)
    # 16 for translation on the fly
    # this values can be logically ORed

MOD_XML='''<?xml version="1.0" encoding="ISO-8859-1"?><module name="Http">'''

def load_spam(filePath):
    """This function is used to load the Spam-policy Tree
    once for all.
    """
    try:
        spamElement=read_domtree_from_uri(filePath)
        return spamElement.documentElement
    except IOError:
        #print sys.exc_info()[0]," unable to open : ",filePath
        return None

SPAM=load_spam("file://"+os.path.abspath(config.get_home())+"/data/spam.xml")

##
## SpamParser class
##

class SpamParser(htmllib.HTMLParser):
    """Class used to handle spam over a Web page"""
    
    def __init__(self,junkbuster_method,format):
        self.junkbuster_method=junkbuster_method
        htmllib.HTMLParser.__init__(self,format)
    
    def anchor_bgn(href,name,type):pass
    def anchor_bgn(self,href,name,type):pass
    def anchor_end(self):pass
    def save_bgn(self):pass
    def save_end(self):pass
    def handle_image(self,source,alt='',ismap='',align='',width=0,height=0):
        """ method which contains all the tests for deciding if an image
        must be trashed or not """
        
        img_server=urlparse.urlparse(source)[1]    
        
        if not img_server:
            return

        # if we have to trash according to server names
        if self.junkbuster_method&1:
            if img_server!=self.server:
                self.spam.append(source)

        # if we have to trash according to domain names
        if self.junkbuster_method&2:
            if img_server[-self.domainnamelength:]!=self.domainname:
                self.spam.append(source)

    def trash_spam(self,html):

        # browse all the images'url to trash (collected while parsing page)
        # and replace them by a blank image
        for url in self.spam:
            rgxp=re.compile('[Ss][Rr][Cc][ \t]*=[ \t]*["\']?'+
                            re.escape(url)+'[ \t]*["\']?',re.M)
            # FIXME tr.png n'existe plus
            imagePath = '"file://'+os.path.abspath(config.get_home())+'/data/tr.png "'
            html=rgxp.sub('src='+imagePath,html)
        if self.junkbuster_method&8:
            sm=SpamMatcher(self.spam_policy)
            return sm.filter(html,self.url,self.url_tuple)
        else:
            return html
        



##
##  SpamMatcher class
##

class SpamMatcher:
    def __init__(self,spampolicytree):
        self.rules={}
        self.instructions=[]
        self.filters=[]
        if doc:
            doc=spampolicytree.ownerDocument
            try:
                for rule in select_many(spampolicytree,'rules/*'):
                    self.rules[rule.getAttributeNS('','name')]=rule
            except SelectException:
                pass
            try:
                for foo in select_many(spampolicytree,'policy/*'):
                    if foo.nodeName in ['allow','trash']:
                        self.instructions.append(foo)
                        continue
                    # here, we know foo.nodeName is 'filter'
                    try:
                        url_match=select_many(foo,'url_match/*')
                    except SelectException:
                        url_match=[]
                    try:
                        replace_by=select_one(foo,'replace_by/text()').data
                    except SelectException:
                        replace_by=''
                    text_match=[]
                    for bar in select_many(foo,'text_match/text()'):
                        text_match.append(bar.data)
                    fh=foo.getAttributeNS('','host')
                    self.filters.append((url_match,text_match,replace_by,fh))
            except SelectException:
                pass

    def test_http_rule(self,tree,url,url_tuple):
        if tree.getAttributeNS('','reverse')!='yes':
            rev=0
        else:
            rev=1
                
        z=tree.getAttributeNS('','url')
        if z and not re.match(z,url,re.VERBOSE|re.M):
            return 0
        pr,h,pat,par,q,f=url_tuple
        z=tree.getAttributeNS('','host')
        if z and not re.match(z,h,re.VERBOSE|re.M):
            return 0
        z=tree.getAttributeNS('','path')
        if z and not re.match(z,pat,re.VERBOSE|re.M):
            return 0
        z=tree.getAttributeNS('','protocol')
        if z and not re.match(z,pr,re.VERBOSE|re.M):
            return 0
        z=tree.getAttributeNS('','param')
        if z and not re.match(z,par,re.VERBOSE|re.M):
            return 0
        z=tree.getAttributeNS('','query')
        if z and not re.match(z,q,re.VERBOSE|re.M):
            return 0
        z=tree.getAttributeNS('','fragid')
        if z and not re.match(z,f,re.VERBOSE|re.M):
            return 0
        return 1
        children=select_many(tree,'*')
        if not len(children):
            return 1^rev
        for foo in children:
            if self.eval_rules_tree(foo,url,url_tuple):
                return 1^rev
        return rev

    def eval_rules_tree(self,tree,url,url_tuple):
        t=tree.nodeName
        if t=='rule':
            return self.test_http_rule(tree,url,url_tuple)
        if tree.getAttributeNS('','reverse')!='yes':
            rev=0
        else:
            rev=1
        if t=='ruleref':
            if not self.test_http_rule(self.rules[tree.getAttributeNS('','rule')],url,url_tuple):
                return rev
            children=select_many(tree,'*')
            if not len(children):
                return rev^1
            else:
                for foo in children:
                    if self.eval_rules_tree(foo,url,url_tuple):
                        return rev^1
                return rev
        if t=='or':
            for rule in select_many(tree,'*'):
                if self.eval_rules_tree(rule,url,url_tuple):
                    return rev^1
            return rev
        # here, we know t=='and'
        for rule in select_many(tree,'*'):
            if not self.eval_rules_tree(rule,url,url_tuple):
                return rev
            return rev^1

    # this returns 1 if the url should be blocked
    def eval_url(self,url,url_tuple):
        for instruction in self.instructions:
            for rules_tree in select_many(instruction,'*'):
                if self.eval_rules_tree(rules_tree,url,url_tuple):
                   if instruction.nodeName=='allow':
                       return 0
                   # here, we know instruction.nodeName=='trash'
                   return 1
        return 0

    # this takes a text, applies filters on it, and returns the filtered text
    def filter(self,text,url,url_tuple):
        for url_match,text_match,replace_by,fh in self.filters:
            if fh:
                if not re.match(fh,url_tuple[1],re.VERBOSE|re.M):
                    continue
            for foo in url_match:
                if self.eval_rules_tree(foo,url,url_tuple):
                    for bar in text_match:
                        rgxp=re.compile(bar,re.VERBOSE|re.M)
                        text=rgxp.sub(replace_by,text)
        return text

##
## gettype : returns the type contained in the Contain-Type field of a string
##           of MIME headers
##

def gettype(headers):
    for line in string.split(headers,'\n'):
        str=string.lower(line[:12])
        if str == 'content-type':
            beg=13
            while line[beg]==' ' or line[beg]=='\t':
                beg=beg+1
            end=beg+1
            while line[end] not in ['\n',';','\r']:
                end=end+1
            return line[beg:end]
    return ''


def domainname(dns):
    """
    returns the domain name part of a DNS
    """
    dot=string.find(dns,'.')
    if dot in [-1,0]:
        return ''
    return dns[dot+1:]



##
## Write back to socket
##

def Write_back_to_socket_f(args) :
    """ function which retrieves all informations concerning the http response
    from the proxy and write them back to the browser (i.e Netscape)
    """

    sock = select_one(args,'socket')
    hi = select_one(args,'http-response')
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    
    s.connect(('localhost',int(sock.getAttributeNS('','port'))))
    s.send('HTTP/1.0 '+hi.getAttributeNS('','errcode')+' '+\
                       hi.getAttributeNS('','errmsg')+'\r\n')
    
    try:
        data = select_one(hi,'headers/text()').data
        s.send(data)
        if data[-1] != '\n' :
            s.send('\r\n\r\n')
        else:
            s.send('\r\n')
        try:
            s.send(select_one(hi,'content/text()').data.encode('ISO-8859-1'))
        except SelectException:
            pass
        s.send('\r\n')
        s.close()
        return args.ownerDocument.createElementNS('','outputs')
    except SelectException:
        return args.ownerDocument.createElementNS('','outputs')

MOD_XML=MOD_XML+'''
<action name='Write_back_to_socket' func='Write_back_to_socket_f'>
  <description lang="en">Send back response to client</description>
  <description lang="fr">Renvoie la rponse au client</description>
  <input use="yes"><match>http-response</match>
  <description lang="en">response to write back</description></input>
  <input><match>socket</match>
  <description lang="en">socket to write the response</description></input>
</action>'''

##
## EXTRACT URL
##

def extract_url_to_text_f(args) :
    a=select_one(args,'http-request')
    url=select_one(a,'url/text()')
    doc=args.ownerDocument
    r=doc.createElementNS('','outputs')
    n=doc.createElementNS('','text')
    r.appendChild(n)
    t=doc.createElementNS('','value')
    n.appendChild(t)
    t.appendChild(doc.createTextNode(url))
    return r

MOD_XML=MOD_XML+"""
<action name='Extract_url' func='extract_url_to_text_f'>
  <description lang='en'>Transform an url into a text</description>
  <description lang='fr'>Transforme une url en un texte</description>
  <input><match>http-request</match>
  <description lang='en'>request element where we could find an url</description></input>
  <output><match>text/value/text()</match>
  <description lang='en'>the extracted url's text</description></output>
</action>"""

##
## NOTE UR 
## 
def note_url_f(args) :
    import time
    a=select_one(args,'http-request')
    url=select_one(a,'url/text()')
    ref=select_many(a,'headers/text()')

    f = open(config.get_home()+'/data/urlvisited.xml','a+')
    f.write('<url url="'+re.sub('<','&lt;',re.sub('&','&amp;',url.data))+'" ')
    f.write('date="'+time.asctime(time.localtime(time.time()))+'"/>\n')
    return args.ownerDocument.createElementNS('','outputs')

MOD_XML=MOD_XML+''' 
<action name="note_url" func="note_url_f">
  <description lang="en">Add the url to the list of visited urls</description>
   <description lang="fr">Ajoute l\'url  la liste des url visites</description>
  <input><match>http-request</match>
  <description lang="en">request matching url to add</description></input>
</action>'''  


##
## PARSE CGI
##

def read_cgi_request_f(args) :
    request=select_one(args,'http-request')
    import cgi
    # read stuff from request
    userinput = {}
    if request.getAttributeNS('','type') == 'post':
        data=select_one(request,'data/text()').data
        userinput = cgi.parse_qs(data)
    if request.getAttributeNS('','type') == 'get':
        url=select_one(request,'url/text()').data
        purl = urlparse.urlparse(url)
        userinput = cgi.parse_qs(purl[4])
    # create output
    doc=args.ownerDocument
    ret=doc.createElementNS('','outputs')
    r=doc.createElementNS('','http-cgi-request')
    ret.appendChild(r)
    u=doc.createElementNS('','url')
    r.appendChild(u)
    u.appendChild(doc.createTextNode(re.sub('\?.*','',select_one(request,'url/text()').data)))
    if userinput:
        for k,val in userinput.items():
            n=doc.createElementNS('','cgi-item')
            n.setAttributeNS('','key',k)
            for v in val:
                n.appendChild(doc.createTextNode(unicode(str(v), 'ISO-8859-1')))
            r.appendChild(n)
    return ret

MOD_XML = MOD_XML + '''
<action name="read_cgi_request" func="read_cgi_request_f">
<description lang="en">Parse a cgi request and output a CGI structure</description>
<description lang="fr">Transforme une requte CGI HTTP en une structure CGI XML
</description>
<input><match>http-request</match>
         <description lang="en">cgi request to parse</description></input>
<output><match>http-cgi-request</match>
         <description lang="en">xml cgi request</description></output>
</action>'''

##
## HTTP REQUEST
##

class HTTPRequest :
    """Read file and parse as an http request"""
    
    def __init__(self, socket) :
        self.socketData = socket.recv(1000)
        #print 'REQUEST |',self.socketData,'|' 
	self.type, self.url, self.httpversion = '', '', ''
        self.purl = ()
	self.headers = []
	self.data = []

    def head(self,line):
        self.type,self.url,self.httpversion=string.split(line)
        self.purl=urlparse.urlparse(self.url)
    def __str__(self) :
	s = 'REQUEST= '+self.type+' '+self.url+' '+self.httpversion
	s = s +'\nHeaders:\n'
	for h in self.headers :
	    s = s + '  '+ h + '\n'
	s = s +'Data:\n'
	for d in self.data :
	    s = s + '  '+d + '\n'
	return s

    def page(self) :
	return self.url[-5:] == '.html' or \
	       self.url[-1:] == '/' or \
	       self.url[-4:] == '.htm' or \
	       self.url[-6:] == '.shtml'

    def get_events(self,foo):
        doc=foo.ownerDocument
        r=doc.createElementNS('','http-request')
        r.setAttributeNS('','type',string.lower(self.type))
        r.setAttributeNS('','http-version',self.httpversion)
        foo.appendChild(r)
        n=doc.createElementNS('','url')
        n.appendChild(doc.createTextNode(self.url))
        r.appendChild(n)
        n=doc.createElementNS('','headers')
        n.appendChild(doc.createTextNode(string.join(self.headers,'\r\n')))
        
        r.appendChild(n)
        n=doc.createElementNS('','data')
        try:
            n.appendChild(doc.createTextNode(latin1_to_utf8(self.raw_data)))
        except AttributeError:
            pass
        
        r.appendChild(n)
        n=doc.createElementNS('','from')
        n.setAttributeNS('','plan','')
        n.setAttributeNS('','cell','')
        r.appendChild(n)
        return r

    def parse(self):
        lines = string.split(self.socketData,'\r\n')

        for line in lines:
            #line=self.getline()
            if not line:
                break
            if not self.type:
                self.head(line)
            else:
                self.headers.append(line)
        if not self.type:
            raise KeyError
        
        if self.type=='POST':
            #self.type='GET'

            
            #self.raw_data=self.rfile.read()
            #print "self.raw_data ",self.raw_data
            self.raw_data = self.socketData
            dataList = string.split(self.raw_data,'\r\n\r\n')
            
            self.raw_data = string.join(dataList[1:],'\r\n')
            
            
            
    def getline(self):
        line=self.rfile.readline()
        
        if not line:
            return None
        if line[-2:]=='\r\n':
            if len(line)>2:
                return line[:-2]
            else:
                return None
        if line[-1:]=='\n':
            if len(line)>1:
                return line[-1:]
            else:
                return None
        return line

def http_parse_request_f(args) :
    import socket
    try:
        proxy=select_one(args,'proxy')
        export_http_proxy(proxy)
    except SelectException:
        pass

    sock=select_one(args,'socket')
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(('localhost',int(sock.getAttributeNS('','port'))))
    req = HTTPRequest(s)
    req.parse()
    s.close()
    foo=args.ownerDocument.createElementNS('','outputs')
    req.get_events(foo)
    return foo

MOD_XML=MOD_XML+'''
<action name='http_parse_request' func='http_parse_request_f'>
  <description lang="en">Parse data on socket and output a structured HTTP request
  </description>
  <description lang="fr">Lit les donnes en entre sur la chaussette et produit
  une requte HTTP structure.</description>
  <input><match>socket</match>
         <description lang="en">socket to read the http request</description></input>
  <input optional="yes"><match>proxy[@type="http"]</match>
  <description lang="en">proxy element, for page requests</description></input>
  <output><match>http-request</match>
         <description lang="en">xml http request</description></output>
</action>'''


##
## HTTP REQUEST
##

##def html2xhtml_f(args) :
##    # get HTML string
##    html = select_one(args,'raw-file/data')
##    data = ''
##    for node in html.childNodes :
##        if node.nodeName == "#cdata-section" :
##            data = node.data
            
##    # parse Html to a DOM tree
##    from narval.ext import HtmlLib
##    doc = args.ownerDocument
##    outputs = doc.createElementNS('','outputs')
##    xhtml = doc.createElementNS('','xhtml')
##    htmldoc = StripXml(HtmlLib.FromHtml(data))
##    xhtml.appendChild(doc.importNode(htmldoc.documentElement,DEEP_COPY))
##    outputs.appendChild(xhtml)
##    return outputs
    
def html2xhtml_f(args) :
    from xml.dom.ext.reader import HtmlLib
    doc = args.ownerDocument
    outputs = doc.createElementNS('','outputs')
    xhtml = doc.createElementNS('','xhtml')
    # get HTML string
    html = select_one(args,'raw-file/data')
    for node in html.childNodes :
        if node.nodeName == "#text" :
            data = node.data
            htmldoc = StripXml(HtmlLib.FromHtml(data,doc))
            xhtml.appendChild(htmldoc.firstChild)

    outputs.appendChild(xhtml)
    return outputs
    

MOD_XML=MOD_XML+'''
<action name='html2xhtml' func='html2xhtml_f'>
  <description lang="en">Tidy HTML and convert to XHTML</description>
  <description lang="fr">Nettoie le HTML et le convertit en XHTML</description>
  <input><match>raw-file/data</match>
  <description lang="en">html file element</description></input>
  <output><match>xhtml</match>
  <description lang="en">produced xhtml element</description></output>
</action>'''


def get_replace(obj):
    list = obj.group()
    str = re.split('&url=',list)
    adr = str[-1]
    adr = string.replace(adr,'%2F','/')
    adr = string.replace(adr,'%3A',':')
    adr = string.replace(adr,'%3F','?')
    adr = string.replace(adr,'%3D','=')
    adr = string.replace(adr,'%2B','+')
    adr = string.replace(adr,'%3B',';')
    return 'href="%s"'%adr



def filter(response,ownerDocument,junkbuster_method):
    doc=ownerDocument
    outputElement=doc.createElementNS('','outputs')
    outputElement.appendChild(response)

    # if jukbuster_method is set to 4 or 0
    if not junkbuster_method&~4:
        return outputElement
    try:
        textElement=select_one(response,'headers/text()')
        if gettype(textElement.data)!='text/html':
            return outputElement
        
        
        bar=select_one(response,'content/text()')
        
        sp=SpamParser(junkbuster_method,formatter.NullFormatter())
        sp.url=select_one(response,'url/text()').data
        sp.url_tuple=urlparse.urlparse(sp.url)
        sp.server=sp.url_tuple[1]
        sp.domainname=domainname(sp.server)
        sp.domainnamelength=len(sp.domainname)
        
        sp.spam=[]
        
        
        sp.spam_policy=SPAM
        # sp.spam_policy=select_one(args,'spam-policy')
        
        sp.handle_image(sp.url)
        sp.feed(bar.data)
        bar.data=sp.trash_spam(bar.data)
       
        pattern = re.compile('href[\t  ]*=[\t ]*["\']?[^\\?>]+\\?lp[^&]*&url=(?P<adr>[^ \t>"\']*)["\']?',re.I)
        bar.data = re.sub(pattern,get_replace,bar.data)
        
        
        return outputElement
    except SelectException:
        import traceback
        traceback.print_exc()
        return outputElement







def findDestLanguage(spokenList,destList):
    for l in spokenList:
        try:
            index = destList.index(l)
            return destList[index]
        except ValueError:
            pass

    return None


# this function takes in arguments the "source language" (Web page's language),
# the list of spoken languages, and returns the translation type ('en_fr' for instance)
def choose_translation_type(sourceLanguage,spokenList):

    # these lists are made according to Altavista possibilities
    if sourceLanguage == 'en':
        destList = ['fr','de','it','es','pt']
    elif sourceLanguage == 'de':
        destList = ['en','fr']
    elif sourceLanguage == 'fr':
        destList = ['en','de']
    elif sourceLanguage == 'it':
        destList = ['en']
    elif sourceLanguage == 'es':
        destList = ['en']
    elif sourceLanguage == 'pt':
        destList = ['en']

    destLanguage = findDestLanguage(spokenList,destList)
    if destLanguage == None:
        return None

    return sourceLanguage+'_'+destLanguage


###############################################################
#         special characters (and translation) list
#
#  " " : "<" : ">" : "#" : "%" : "{" : "}" : "|"
#  "\\" : "^" : "~" : "[" : "]" : "`" : ";"
#  "/" : "?" : ":" : "@" : "=" : "&" : "$";
#  "%20" : "%3C" : "%3E" : "%23" : "%25"
#  "%7B" : "%7D" : "%7C" : "%5C" : "%5E
#  "%2F" : "%3F" : "%3A" : "%40" : "%3D"
#  "%26" : "%24" )
###############################################################


def build_redirected_url(proxyString,lang,url):
    url = string.replace(url,'/','%2F')
    url = string.replace(url,':','%3A')
    url = string.replace(url,'?','%3F')
    url = string.replace(url,'=','%3D')
    url = string.replace(url,'#','%23')
    url = string.replace(url,';','%3B')
    url = string.replace(url,'+','%2B')

    #http://www.lycos.fr/cgi-bin/pursuit?matchmode=and&mtemp=main&etemp=error&query=astronomie&cat=fr
    
    return proxyString+'lp='+lang+'&url='+url


def replace_url(url):
    list = string.split(url,'?')
    base = list.pop(0)
    # if there is more than one '?' in the original address
    right_part = string.join(list,'?')
    
    
    if len(right_part) > 0:
        # get the "spec_list"
        spec_list = string.split(right_part,'&')
        # remove "BabelFishForm=GET" if needed
        if right_part == 'BabelFishForm=GET':
            spec_list.pop(0)
            right_part = string.join(spec_list,'&')

        
        # retrieve the "url" part
        list = string.split(right_part,'url=')
        right_list = list[1:]
                
        # if 'url=' appear in the "real" address
        right_part = string.join(right_list,'url=')

        # replace special characters
        right_part = string.replace(right_part,'/','%2F')
        right_part = string.replace(right_part,':','%3A')
        right_part = string.replace(right_part,'?','%3F')
        right_part = string.replace(right_part,'=','%3D')
        right_part = string.replace(right_part,'#','%23')
        right_part = string.replace(right_part,';','%3B')
        right_part = string.replace(right_part,'+','%2B')
        right_part = string.replace(right_part,'&','%3F')        
        # right_part = string.join(spec_list)
        
        if len(right_part) > 0:
            right_part = list[0]+'url='+right_part
        else:
            right_part = list[0]

        url = base+'?'+right_part

    return url

def get_and_filter_f(args):
    """ function which retrieves a parsed request, makes it,
    and filter according to the junkbuster_method's value
    """

    # retrieve the request
    event=select_one(args,'http-request')
    url=select_one(event,'url/text()').data

    # if special characters, replace them
    url = replace_url(url)

    # urlparse format :
    # scheme://netloc/path;parameters?query#fragment
    ask_url=urlparse.urlparse(url)
    netloc=ask_url[1]
    urlList=string.split(netloc,'.')
    
    # prepare output
    doc=args.ownerDocument
    
    r=doc.createElementNS('','outputs')
    h=doc.createElementNS('','http-response')
    r.appendChild(h)
    nu=doc.createElementNS('','url')
    nu.appendChild(doc.createTextNode(url))
    h.appendChild(nu)
    ne=doc.createElementNS('','errmsg')
    h.appendChild(ne)
    nh=doc.createElementNS('','headers')
    h.appendChild(nh)
    nc=doc.createElementNS('','content')
    h.appendChild(nc)

    # find which type of junkbuster is on
    try:
        jbmElement=select_one(args,'junkbuster-method')
        junkbuster_method=int(jbmElement.getAttributeNS('','value'))
    except SelectException:
        junkbuster_method=0
    
    # if junkbuster is on
    if junkbuster_method&4:
        import os
        sm=SpamMatcher(SPAM)
        if sm.eval_url(url,ask_url):
            tapoueh=open(os.path.abspath(config.get_home())+"/data/tr.png","r")
            data=tapoueh.read()
            nc.appendChild(doc.createTextNode(data))
            tapoueh.close()
            str_headers='Content-Length: '+str(len(data))+'\r\n'
            str_headers=str_headers+'Connection: close\r\n'
            str_headers=str_headers+'Content-Type: image/png\r\n'
            h.setAttributeNS('','errcode','200')
            ne.appendChild(doc.createTextNode('OK'))
            nh.appendChild(doc.createTextNode(str_headers))
            outputElement=filter(h,args.ownerDocument,junkbuster_method)
            return outputElement

    
    # send the request
    # Use a proxy if asked in arguments
    try:
        proxy=select_one(args,'proxy')
        export_http_proxy(proxy)
        zh=httplib.HTTP(proxy.getAttributeNS('','hostname'),
                        int(proxy.getAttributeNS('','port')))
        
        zh.putrequest(string.upper(event.getAttributeNS('','type')),url)

    # if no proxy
    except SelectException:
        try:
            zh = httplib.HTTP(ask_url[1])
            ask_url = urlparse.urlunparse( ('', '', ask_url[2], ask_url[3],
                                            ask_url[4], ask_url[5]) )
            zh.putrequest(string.upper(event.getAttributeNS('','type')), ask_url)
        except:
            print "impossible to connect to \"",ask_url[1],"\""
            return r


    # send all the headers
    for head in select_many(event,'headers/text()'):
        for a in string.split(head.data,'\015\012'):
            try:
                (k,v)=tuple(string.split(a,': '))
                zh.putheader(k,v)
            except ValueError:
                pass
            
    # send a blank line
    zh.endheaders()

    # send more data if needed by the request
    for foo in select_many(event,'data/text()'):
        zh.send(foo.data)
        
    # get reply
    errcode,errmsg,headers=zh.getreply()
    str_headers=str(headers)

    # if no problems
    if errcode==200:
        f=zh.getfile()
        type = gettype(str_headers)
        # if not an image, try to find the language and translate it if needed
        data = f.read()
        if type == 'text/html':
            if lg_factory and junkbuster_method&16:
                try:
                    spokenElement = select_one(args,'spoken-languages')
                    languageList=spokenElement.childNodes
                    
                    # create a list containing all the spoken languages
                    spokenList = []
                    for languageElement in languageList:
                        lang = languageElement.getAttributeNS('','code')
                        spokenList.append(lang)

                    # determine the page's language
                    # sourceLanguage = lg_factory.identify_language_from_url(url)
                    sourceLanguage = lg_factory.identify_language_from_buffer(data)
                    ask_url = urlparse.urlparse(url)
                    
                    
                    try:
                        languageElement=select_one(spokenElement,'language[@code="%s"]'%sourceLanguage)
                    except SelectException:
                        #if not spoken
                        lang=choose_translation_type(sourceLanguage,spokenList) 
                    
                        # %3A <-> ':', %2F <-> '/'
                        if lang != None:
                            url = build_redirected_url('http://babel.altavista.com/urltrurl?',lang,url)
                            #url = build_redirected_url('http://fr.altavista.com/scripts/eusearchengine.dll?',lang,url)
                            ask_url=urlparse.urlparse(url)
                            netloc = ask_url[1]
                            
                            # now retrieve the new page's data
                            #if proxy # FIXME, avoid select proxy (done above)
                            try:
                                proxy=select_one(args,'proxy')
                                
                                zh=httplib.HTTP(proxy.getAttributeNS('','hostname'),
                                                int(proxy.getAttributeNS('','port')))
                                
                                zh.putrequest(string.upper(event.getAttributeNS('','type')),url)
                            #else
                            except SelectException:
                                try:
                                    zh = httplib.HTTP(ask_url[1])
                                    ask_url = urlparse.urlunparse( ('', '', ask_url[2], ask_url[3],
                                                                    ask_url[4], ask_url[5]) )
                                    zh.putrequest(string.upper(event.getAttributeNS('','type')), ask_url)
                                except :
                                    print sys.exc_info()[0]," -> impossible to connect to \"",ask_url[1],"\""
                                    return r
                            
                            # put all headers
                            for head in select_many(event,'headers/text()'):
                                for a in string.split(head.data,'\015\012'):
                                    (k,v)=tuple(string.split(a,': '))
                                    # babel.altavista.com
                                    if k == 'Host':
                                        v = netloc
                                    zh.putheader(k,v)


                            zh.endheaders()

                            # send data 
                            for foo in select_many(event,'data/text()'):
                                zh.send(foo.data)
                                
                            # get reply
                            errcode,errmsg,headers=zh.getreply()
                            
                            str_headers=str(headers)
                            if errcode != 200:
                                data='<p>The server returned the following error : </p><p><strong>'+\
                                      str(errcode)+' - '+errmsg+\
                                      '</strong></p><p>And gave the following headers :</p><p><pre>'+\
                                      str_headers+'</pre></p>'
                                h.setAttributeNS('','errcode',str(errcode))
                                ne.appendChild(doc.createTextNode(errmsg))
                                nh.appendChild(doc.createTextNode(str_headers))
                                nc.appendChild(doc.createTextNode(data))
                                
    
                                outputElement=filter(h,args.ownerDocument,junkbuster_method)
                                return outputElement

                            f = zh.getfile()
                            data = f.read()
                except SelectException:
                    pass
        f.close()
    # if an error occured
    else:
        data='<p>The server returned the following error : </p><p><strong>'+\
              str(errcode)+' - '+errmsg+\
              '</strong></p><p>And gave the following headers :</p><p><pre>'+\
              str_headers+'</pre></p>'
    
    h.setAttributeNS('','errcode',str(errcode))
    ne.appendChild(doc.createTextNode(errmsg))
    nh.appendChild(doc.createTextNode(str_headers))
    nc.appendChild(doc.createTextNode(data))
    
    
    outputElement=filter(h,args.ownerDocument,junkbuster_method)
    return outputElement

MOD_XML=MOD_XML+'''
<action name="get_and_filter" func="get_and_filter_f">
  <description lang="en">Filter out web junk (ads, cookies, etc.)</description>
  <description lang="fr">Filtre les ordures du web (pubs, cookies, etc.)</description>
  <input><match>http-request[url/text()][headers/text()]</match>
  <description lang="en">http request to filter</description></input>
  <input optional="yes"><match>proxy[@type="http"]</match>
  <description lang="en">proxy element, for page requests</description></input>
  <input optional="yes"><match>spam-policy</match>
  <description lang="en">spam policy element</description></input>
  <input optional="yes"><match>junkbuster-method</match>
  <description lang="en">junkbuster configuration element</description></input>
  <input optional="yes"><match>spoken-languages</match>
  <description lang="en">node containing user\'s spoken languages</description></input>
  <output><match>http-response</match>
  <description lang="en"></description></output>
</action>'''

def export_http_proxy(proxy):
    from os import environ
    environ['http_proxy'] = 'http://%s:%s' % (proxy.getAttributeNS('','hostname'),
                                              proxy.getAttributeNS('','port'))

MOD_XML=MOD_XML+'</module>'

if __name__ == "__main__" :
    print MOD_XML
    from xml.dom.ext.reader import Sax2
    doc=Sax2.FromXml(MOD_XML)
    print doc.documentElement
