# Copyright (c) 2000 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import re
from narval.lib import *
import _ldap

LDAP_NS = ''

MOD_XML = """<?xml version='1.0' encoding='iso-8859-1'?><module name='LDAP'>"""


def modify_entry_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(LDAP_NS,'outputs')

    ldap = select_one(args,'ldap-server')
    host = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'host'))
    port = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'port'))
    # connect to the LDAP server
    if port == '':
        l = _ldap.open(host)
    else:
        l = _ldap.open(host,int(port))

    l.simple_bind("","")
    dn = select_one(args,'asked-dn/text()').data
    modif_nodes = select_one(args,'ldap-modifs').childNodes
    modif_list = []
    for modif in modif_nodes:
        attr_value = utf8_to_latin1(modif.getAttributeNS(LDAP_NS,'value'))
        attr_name = utf8_to_latin1(modif.getAttributeNS(LDAP_NS,'name'))
        if modif.nodeName == 'tags':
            continue
        elif modif.nodeName in ['add-attr','add-entry']:
            modif_list.append((_ldap.MOD_ADD,attr_name,attr_value))
        elif modif.nodeName == 'remove-attr':
            modif_list.append((_ldap.MOD_DELETE,attr_name,attr_value))
        elif modif.nodeName == 'replace-attr':
            modif_list.append((_ldap.MOD_REPLACE,attr_name,attr_value))
        else:
            print "Unknown LDAP modifictation type : ",modif.nodeName
    l.modify_s(dn,modif_list)
    l.unbind_s()
    return outputs

MOD_XML = MOD_XML + \
          """
          <action name='modify_entry' func='modify_entry_f'>
          <description lang='en'>modifies an entry of the directory</description>
          <input><match>ldap-server[@host]</match></input>
          <input use='yes'><match>asked-dn/text()</match></input>
          <input use='yes'><match>ldap-modifs</match></input>
          </action>
          """


def consult_directory_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(LDAP_NS,'outputs')
    entry_list = doc.createElement('ldap-entries')
    outputs.appendChild(entry_list)
    
    ldap = select_one(args,'ldap-server')
    host = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'host'))
    port = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'port'))
    # connect to the LDAP server
    if port == '':
        l = _ldap.open(host)
    else:
        l = _ldap.open(host,int(port))

    l.simple_bind("","")
    asked_dn = select_one(args,'asked-dn')
    dn = utf8_to_latin1(select_one(asked_dn,'text()').data)
    
    import string
    type = string.upper(utf8_to_latin1(asked_dn.getAttributeNS(LDAP_NS,'scope-type')))
    if type == 'ONELEVEL':
        scope_type = _ldap.SCOPE_ONELEVEL
    elif type == 'BASE':
        scope_type = _ldap.SCOPE_BASE
    elif type in ['SUBTREE','']:
        scope_type = _ldap.SCOPE_SUBTREE
    else:
        print "Warning : unknown scope-type ["+type+"], using default : [SUBTREE]"
        scope_type = _ldap.SCOPE_SUBTREE
    
    
    objectclass = asked_dn.getAttributeNS(LDAP_NS,'objectclass')
    if objectclass == '':
        objectclass = '*'
    entries = l.search_s(dn,scope_type,"objectclass="+objectclass)

    for entry in entries:
        entry_node = doc.createElementNS(LDAP_NS,'entry')
        dn_node = doc.createElementNS(LDAP_NS,'object-dn')
        dn_node.appendChild(doc.createTextNode(entry[0]))
        entry_node.appendChild(dn_node)
        for key,values in entry[1].items():
            key_element = doc.createElementNS(LDAP_NS,key)
            key_element.appendChild(doc.createTextNode(string.join(values,',')))
            entry_node.appendChild(key_element)
        entry_list.appendChild(entry_node)

    l.unbind_s()
    return outputs


MOD_XML = MOD_XML + \
          """
          <action name='consult_directory' func='consult_directory_f'>
          <description lang='en'>searches for a dn on the LDAP directory</description>
          <input><match>ldap-server[@host]</match></input>
          <input use='yes'><match>asked-dn/text()</match></input>
          <output><match>ldap-entries</match></output>
          </action>
          """

def ldap_url_search_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(LDAP_NS,'outputs')
    result_node = doc.createElementNS(LDAP_NS,'ldap-entries')
    outputs.appendChild(result_node)

    ldap = select_one(args,'ldap-server')
    host = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'host'))
    port = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'port'))
    # connect to the LDAP server

    if port == '':
        l = _ldap.open(host)
    else:
        l = _ldap.open(host,int(port))

    l.simple_bind("","")
    ldap_url = utf8_to_latin1(select_one(args,'ldap_url/text()').data)
    entries = l.url_search_s(ldap_url)
    
    for entry in entries:
        entry_node = doc.createElementNS(LDAP_NS,'entry')
        dn_node = doc.createElementNS(LDAP_NS,'object-dn')
        dn_node.appendChild(doc.createTextNode(entry[0]))
        entry_node.appendChild(dn_node)
        for key,values in entry[1].items():
            key_element = doc.createElementNS(LDAP_NS,key)
            key_element.appendChild(doc.createTextNode(string.join(values,',')))
            entry_node.appendChild(key_element)
        entry_list.appendChild(entry_node)

    l.unbind_s()
    
    return outputs
    
    
                                  

MOD_XML = MOD_XML + \
          """
          <action name='ldap_url_search' func='ldap_url_search_f'>
          <description lang='en'>searches in the LDAP directory using LDAP url syntax</description>
          <input><match>ldap-server[@host]</match></input>
          <input><match>ldap-url</match></input>
          <output><match>ldap-entries</match></output>
          </action>
          """


def add_agent_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(LDAP_NS,'outputs')
    ldap = select_one(args,'ldap-server')
    host = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'host'))
    port = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'port'))
    # connect to the LDAP server
    if port == '':
        l = _ldap.open(host)
    else:
        l = _ldap.open(host,int(port))

    l.simple_bind("","")
    attr_list = []

    import string
    agent_def = select_one(args,'agent-def')
    agent_name = utf8_to_latin1(select_one(agent_def,'name/text()').data)
    dn = 'cn=%s ,ou=assistants, dc=logilab, dc=fr'%agent_name
    
    address = string.split(utf8_to_latin1(select_one(agent_def,'address/text()').data),':')
    host = address[0]
    port = address[1]
    attr_list.append(('objectclass','narvalassistant'))
    attr_list.append(('host',host))
    attr_list.append(('port',port))
    attr_list.append(('cn',agent_name))
    agent_owner = utf8_to_latin1(select_one(agent_def,'owner/name/text()').data)
    attr_list.append(('assistantof',agent_owner))

    # owner_mail = select_one(agent_def,'owner/mail/text()').data
    attr_list.append(('status','off'))
    try:
        desc = utf8_to_latin1(select_one(agent_def,'description/text()').data)
        attr_list.append(('description',desc))
    except SelectException:
        pass
    # sends the request to the LDAP server
    l.add_s(dn,attr_list)
    l.unbind()
    return outputs

MOD_XML = MOD_XML + \
          """
          <action name='add_agent' func='add_agent_f'>
          <description lang='en'>adds an agent in the LDAP directory</description>
          <input><match>ldap-server[@host]</match></input>
          <input optional='yes'><match>organization</match></input>
          <input outdates='yes'><match>agent-def</match></input>
          </action>
          """


def add_person_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(LDAP_NS,'outputs')
    ldap = select_one(args,'ldap-server')
    host = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'host'))
    port = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'port'))
    # connect to the LDAP server
    if port == '':
        l = _ldap.open(host)
    else:
        l = _ldap.open(host,int(port))

    l.simple_bind("","")
    attr_list = []

    import string
    person_def = select_one(args,'person-def')
    person_name = utf8_to_latin1(select_one(person_def,'name/text()').data)
    dn = 'cn=%s ,ou=people, dc=logilab, dc=fr'%agent_name

    name_data = string.slpit(person_name)
    attr_list.append(('objectclass','person')) 
    attr_list.append(('sn',name_data[1]))
    attr_list.append(('givenname',name_data[0]))
    attr_list.append(('cn',person_name))
    
    title = utf8_to_latin1(select_one(person_def,'title/text()').data)
    attr_list.append(('title',title))
    mail_list = None
    for mail_node in select_many(person_def,'email_addr/text()'):
        if mail_list:
            mail_list = mail_list+', '+utf8_to_latin1(select_one(mail_node,'text()').data)
        else:
            mail_list = utf8_to_latin1(select_one(mail_node,'text()').data)
    attr_list.append(('mail',mail_list))
    # sends the request to the LDAP server 
    l.add_s(dn,attr_list)
    l.unbind()
    return outputs

MOD_XML = MOD_XML + \
          """
          <action name='add_person' func='add_person_f'>
          <description lang='en'>adds a person in the LDAP directory</description>
          <input><match>ldap-server[@host]</match></input>
          <input optional='yes'><match>organization</match></input>
          <input use='yes'><match>person-def</match></input>
          </action>
          """


def remove_entry_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(LDAP_NS,'outputs')
    ldap = select_one(args,'ldap-server')
    host = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'host'))
    port = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'port'))
    # connect to the LDAP server
    if port == '':
        l = _ldap.open(host)
    else:
        l = _ldap.open(host,int(port))

    l.simple_bind("","")
    attr_list = []
    dn = select_one(args,'asked-dn/text()').data

    l.delete_s(dn)
    l.unbind()
    return outputs


MOD_XML = MOD_XML + \
          """
          <action name='remove_entry' func='remove_entry_f'>
          <description lang='en'>removes an entry from the directory</description>
          <input><match>ldap-server[@host]</match></input>
          <input use='yes'><match>asked-dn/text()</match></input>
          </action>
          """

def remove_assistant_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(LDAP_NS,'outputs')
    ldap = select_one(args,'ldap-server')
    host = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'host'))
    port = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'port'))
    # connect to the LDAP server
    if port == '':
        l = _ldap.open(host)
    else:
        l = _ldap.open(host,int(port))

    l.simple_bind("","")
    attr_list = []
    agent_name = select_one(args,'agent-name/text()').data
    dn = 'cn=%s, ou=assistants, dc=logilab, dc=fr'%agent_name
    l.delete_s(dn)
    l.unbind()
    return outputs


MOD_XML = MOD_XML + \
          """
          <action name='remove_assistant' func='remove_assistant_f'>
          <description lang='en'>removes an entry from the directory</description>
          <input><match>ldap-server[@host]</match></input>
          <input use='yes'><match>agent-name/text()</match></input>
          </action>
          """

def current_status(agent_name,ldap_object):
    dict =ldap_object.search_s("cn=%s, ou=assistants, dc=logilab, dc=fr"%agent_name, _ldap.SCOPE_BASE, "objectclass=*")[0][1]
    
    return (dict['status'][0],dict['host'][0],dict['port'][0])

    
    
def assistant_turn_on_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(LDAP_NS,'outputs')
    ldap = select_one(args,'ldap-server')
    host = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'host'))
    port = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'port'))
    # connect to the LDAP server
    if port == '':
        l = _ldap.open(host)
    else:
        l = _ldap.open(host,int(port))

    l.simple_bind("","")
    attr_list = []

    listen_element = select_one(args,'listen-on')
    assistant_port = utf8_to_latin1(listen_element.getAttributeNS(LDAP_NS,'port'))
    assistant_host = utf8_to_latin1(listen_element.getAttributeNS(LDAP_NS,'host'))

    agent_name = utf8_to_latin1(select_one(args,'agent-name/text()').data)
    status = current_status(agent_name,l)
    if status == ('on',assistant_port,assistant_host):
        print "The assistant ",agent_name," is already connected"
    else:
        dn = 'cn=%s, ou=assistants, dc=logilab, dc=fr'%agent_name
        modif_list = []
        modif_list.append((_ldap.MOD_REPLACE,'status','on'))
        
        modif_list.append((_ldap.MOD_REPLACE,'host',assistant_host))
        modif_list.append((_ldap.MOD_REPLACE,'port',assistant_port))
        l.modify_s(dn,modif_list)
        
    l.unbind()
    return outputs


MOD_XML = MOD_XML + \
          """
          <action name='assistant_turn_on' func='assistant_turn_on_f'>
          <description lang='en'>specifies in the directory that an assistant is connected</description>
          <input><match>ldap-server[@host]</match></input>
          <input><match>listen-on[@host and @port]</match></input>
          <input><match>agent-name/text()</match></input>
          </action>
          """
    
def assistant_turn_off_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(LDAP_NS,'outputs')
    ldap = select_one(args,'ldap-server')
    host = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'host'))
    port = utf8_to_latin1(ldap.getAttributeNS(LDAP_NS,'port'))
    # connect to the LDAP server
    if port == '':
        l = _ldap.open(host)
    else:
        l = _ldap.open(host,int(port))

    l.simple_bind("","")
    attr_list = []
    
    agent_name = utf8_to_latin1(select_one(args,'agent-name/text()').data)
    status = current_status(agent_name,l)
    if status == 'off':
        print "The assistant ",agent_name," is not connected"
    else:
        dn = 'cn=%s, ou=assistants, dc=logilab, dc=fr'%agent_name
        l.modify_s(dn,[(_ldap.MOD_REPLACE,'status','off')])
    l.unbind()
    return outputs


MOD_XML = MOD_XML + \
          """
          <action name='assistant_turn_off' func='assistant_turn_off_f'>
          <description lang='en'>specifies in the directory that an assistant is disconnected </description>
          <input><match>ldap-server[@host]</match></input>
          <input use='yes'><match>agent-name/text()</match></input>
          </action>
          """


MOD_XML = MOD_XML + "</module>"


if __name__ == "__main__" :
    from xml.dom.ext.reader import Sax2
    print MOD_XML
    doc=Sax2.FromXml(MOD_XML)
    print doc.documentElement
