# Copyright (c) 2000 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

__revision__ = "$Id: xmlrep.py,v 1.44 2001/10/10 09:47:29 syt Exp $"


from narval.lib import *
from Ft.Lib import cDomlette
from narval.dom.ALDomletteReader import ALDomletteReader
import urllib, urlparse
import types, string

MOD_XML="""<?xml version='1.0' encoding='ISO-8859-1'?>
<module name='xmlrep'>

<action name='get_xslt_result' func='apply_xslt_f'>
<description lang='en'>apply an xsl transform to an xml element</description>
<input id='repository'><match>xml-repository[@type]/text()</match>
<description lang='en'>the repository which contain the element to transform</description></input>
<input  id='transform'><match>transform</match>
<description lang='en'>the xsl tranform</description></input>
<input optional='yes' list='yes' id='additional-node'>
<description lang='en'>optional additional node</description></input>
<output list='yes' optional='yes' id='result'>
<description lang='en'>result from the transforms</description></output>
</action>

<action name='write_xslt_result' func='apply_xslt_f'>
<description lang='en'>apply an xsl transform to an xml element and write the result to a file</description>
<input id='repository'><match>xml-repository[@type]/text()</match></input>
<description lang='en'>the repository which contain the element to transform</description>
<input><match>url</match>
<description lang='en'>url for the result file</description></input>
<input id='transform'><match>transform</match>
<description lang='en'>the xsl tranform</description></input>
<input optional='yes' list='yes' id='additional-node'>
<description lang='en'>optional additional node</description></input>
</action>

<action name='get_node' func='apply_xpath_f'>
<description lang='en'>return a single node matching xpath in a xml repository</description>
<input id='repository'><match>xml-repository[@type]/text()</match>
<description lang='en'>xml repository</description></input>
<input id='location'><match>xpath-location/text()</match>
<description lang='en'>xpath locating the node</description></input>
<output id='result' list='no' optional='yes'>
<description lang='en'>the selected node</description></output>
</action>

<action name='get_nodes' func='apply_xpath_f'>
<description lang='en'>return one ore more nodes matching xpath</description>
<input id='repository'><match>xml-repository[@type]/text()</match>
<description lang='en'>xml repository</description></input>
<input id='location'><match>xpath-location/text()</match>
<description lang='en'>xpath locating the nodes</description></input>
<output id='result' list='yes' optional='yes'>
<description lang='en'>the selected nodes</description></output>
</action>

<action name='xpath_expression' func='apply_xpath_f'>
<description lang='en'>return a node with text child representing the xpath
expression result</description>
<input id='repository'><match>xml-repository[@type]/text()</match>
<description lang='en'>xml repository</description></input>
<input><match>xpath-expression/text()</match>
<description lang='en'>xpath to evaluate</description></input>
<output><match>expression-result</match>
<description lang='en'>node with a text node containing the result</description>
</output>
</action>

<action name='delete_nodes' func='delete_node_f'>
<description lang='en'>delete all nodes matching xpath location</description>
<input id='repository'><match>xml-repository[@type]/text()</match>
<description lang='en'>xml repository to update</description></input>
<input id='location'><match>xpath-location/text()</match>
<description lang='en'>xpath designing the nodes to delete</description></input>
</action>

<action name='insert_node' func='insert_node_f'>
<description lang='en'>insert a new node to the xpath location</description>
<input id='repository'><match>xml-repository[@type]/text()</match>
<description lang='en'>xml repository to update</description></input>
<input id='location'><match>xpath-location/text()</match>
<description lang='en'>xpath designing the place where the node will be
insert</description></input>
<input id='inserted-node'>
<description lang='en'>node to insert</description></input>
<input optional='yes'><match>position/text()</match>
<description lang='en'>optional node with an integer I as text, saying to
insert the node as child number I</description></input>
</action>

<action name='replace_node' func='replace_node_f'>
<description lang='en'>replace a node designed with xpath by a given node</description>
<input id='repository'><match>xml-repository[@type]/text()</match>
<description lang='en'>xml repository to update</description></input>
<input optional='yes' id='location'><match>xpath-location/text()</match>
<description lang='en'>xpath designing the node to replace</description></input>
<input optional='yes' list='yes'><match>discriminant-attribute/text()</match>
<description lang='en'>optional node with an attribute name as text child. The
node will replace a node from the xml repository which as the same attribute value
</description></input>
<input id='inserted-node'>
<description lang='en'>new node</description></input>
</action>

</module>"""



class FileLock:

    def __init__(self):
        self.file_dict={}
        self.lock = threading.Lock()
        

    def acquire(self,file_name):
        file_name = expand_vars(file_name)
        self.lock.acquire()
        if not self.file_dict.has_key(file_name):
            self.file_dict[file_name]=threading.Lock()
        self.file_dict[file_name].acquire()
        self.lock.release()
        
    def release(self,file_name):
        file_name = expand_vars(file_name)
        if not self.file_dict.has_key(file_name):
            print "ERROR : no lock on file ",file_name
        else:
            self.file_dict[file_name].release()
            
xmlrep_lock = FileLock()

def apply_xslt_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    stylesheet = select_one(args,"transform[@input-id='transform']").firstChild
    rep = select_one(args,"xml-repository[@input-id='repository']")
    doc_uri = rep.firstChild.data
    doc_type = rep.getAttributeNS('',"type")
    additional_nodes = select_many(args,"*[@input-id='additional-node']")
    additional_nodes = additional_nodes + select_many(args,"@*")
    if doc_type == 'plain-file' :
        rw_module = None
    else :
        rw_module = module(doc_type)
    try:
        output_uri = select_one(args,"url/text()").data
    except:
        output_uri = None

    result = _process_xslt(doc_uri,rw_module,additional_nodes,stylesheet,doc)

    if output_uri:
        write_tree_and_release(output_uri,rw_module,result,ALDomletteReader())
    else:
        out.appendChild(result)
        ALDomletteReader().releaseNode(result)
    return out


def apply_xpath_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    rep = select_one(args,"xml-repository[@input-id='repository']")
    doc_uri = rep.firstChild.data
    doc_type = rep.getAttributeNS('',"type")
    if doc_type == 'plain-file' :
        rw_module = None
    else :
        rw_module = module(doc_type)
    try:
        xpath = select_one(args,"xpath-location/text()").data
    except:
        xpath = select_one(args,"xpath-expression/text()").data

    
    d,result,reader = _evaluate_xpath(doc_uri,rw_module,xpath)
    if d == None:
        Err = doc.createElementNS('','error')
        Err.setAttributeNS('','type','No such xml repository '+doc_uri)
        out.appendChild(Err)
        return out
    if result == None:
        Err = doc.createElementNS('','error')
        Err.setAttributeNS('','type','No such node '+xpath+' in xml repository ')
        out.appendChild(Err)
        return out
    # ?
    if type(result) == types.ListType:
        for elt in result:
            out.appendChild(doc.importNode(elt,DEEP_COPY))
    else:
        node = doc.createElementNS('','expression-result')
        out.appendChild(node)
        tnode = doc.createTextNode(str(result))
        node.appendChild(tnode)
        
    reader.releaseNode(d)                    
        
    return out


def delete_node_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    rep = select_one(args,"xml-repository[@input-id='repository']")
    doc_uri = rep.firstChild.data
    doc_type = rep.getAttributeNS('',"type")
    if doc_type == 'plain-file' :
        rw_module = None
    else :
        rw_module = module(doc_type)

    xpath = select_one(args,"xpath-location/text()").data

    d,result,reader = _evaluate_xpath(doc_uri,rw_module,xpath,xml_reader = cDomlette.RawExpatReader)

    for elt in result:
        elt.parentNode.removeChild(elt)
        reader.releaseNode(elt)
    write_tree_and_release(doc_uri,rw_module,d,reader)
    return out


def insert_node_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    rep = select_one(args,"xml-repository[@input-id='repository']")
    doc_uri = rep.firstChild.data
    doc_type = rep.getAttributeNS('',"type")
    if doc_type == 'plain-file' :
        rw_module = None
    else :
        rw_module = module(doc_type)
    inserted_node = select_one(args,"*[@input-id='inserted-node']")
    xpath = select_one(args,"xpath-location/text()").data

    d,result,reader = _evaluate_xpath(doc_uri,rw_module,xpath)
    new_node = d.importNode(inserted_node,DEEP_COPY)
    new_node.removeAttributeNS('','eid')
    new_node.removeAttributeNS('','input-id')
    new_node.removeAttributeNS('','timestamp')
    new_node.removeAttributeNS('','from_plan')
    new_node.removeAttributeNS('','from_step')
    for nd in select_many(new_node,'tags') :
        new_node.removeChild(nd)
    
    try:
        position = int(select_one(args,"position/text()").data)
        result[0].insertBefore(new_node,result[0].childNodes[position])
    except:
        result[0].appendChild(new_node)

    write_tree_and_release(doc_uri,rw_module,d,reader)
    return out

def replace_node_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    rep = select_one(args,"xml-repository[@input-id='repository']")
    doc_uri = rep.firstChild.data
    doc_type = rep.getAttributeNS('',"type")
    if doc_type == 'plain-file' :
        rw_module = None
    else :
        rw_module = module(doc_type)
    inserted_node = select_one(args,"*[@input-id='inserted-node']")
    try:
        xpath = select_one(args,"xpath-location/text()").data
        key=None
    except:
        key = select_many(args,"discriminant-attribute/text()")

    if key:
        xpath_build = []
        for k in key:
            key_value = select_one(inserted_node,'@'+k.data).value
            xpath_build.append('@%s="%s"'%(k.data,key_value))
        xpath = '//%s[%s]'%(inserted_node.nodeName,string.join(xpath_build,' and '))

    d,result,reader = _evaluate_xpath(doc_uri,rw_module,xpath)
    new_node = d.importNode(inserted_node,DEEP_COPY)
    new_node.removeAttributeNS('','eid')
    new_node.removeAttributeNS('','input-id')
    new_node.removeAttributeNS('','timestamp')
    new_node.removeAttributeNS('','from_plan')
    new_node.removeAttributeNS('','from_step')
    for nd in select_many(new_node,'.//tags'):
        new_node.removeChild(nd)
    
    result[0].parentNode.replaceChild(new_node,result[0])

    write_tree_and_release(doc_uri,rw_module,d,reader)

    reader.releaseNode(result[0])
    return out




#################################################################
#### Utility functions to be called by action stubs #############


def module(type) :
    from re import sub
    mod = sub('/', '_', type)
    return load_extension_module('xmlrep',mod)


def _process_xslt(doc_uri, rw_module, additional_nodes, stylesheet, dom_output_document):
    xmlrep_lock.acquire(doc_uri)
    try:
        try :
            from xml.xslt.Processor import Processor
            from xml.xslt import DomWriter
            reader = ALDomletteReader()
            processor = Processor()
            xpath_lock.acquire()
            processor.appendStylesheetString(element_to_string(stylesheet))
            xpath_lock.release()
            writer = DomWriter.DomWriter(dom_output_document)
            
            if rw_module :
                doc = rw_module.read(normalize_url(doc_uri)[0])
            else :
                doc = reader.fromUri(normalize_url(doc_uri)[0])
                    
            arg_node = doc.createElementNS('','arguments')
            arg_node.appendChild(doc.documentElement)
            doc.appendChild(arg_node)
            for n in additional_nodes:
                if n.nodeType == n.ATTRIBUTE_NODE :
                    arg_node.setAttributeNS('',n.nodeName,n.value)
                else :
                    arg_node.appendChild(doc.importNode(n,DEEP_COPY))

            xpath_lock.acquire()
            result = processor.runNode(doc,1,{},writer)
            xpath_lock.release()
            reader.releaseNode(doc)
        except :
            import traceback
            traceback.print_exc()
    finally:
        xmlrep_lock.release(doc_uri)
        return result


def _evaluate_xpath(doc_uri,rw_module,xpath,xml_reader=ALDomletteReader):
    doc,result,reader = None, None, None
    xmlrep_lock.acquire(doc_uri)
    try:
        #from xml.xpath import Evaluate
        if rw_module :
            reader = ALDomletteReader()
            doc = rw_module.read(normalize_url(doc_uri)[0])
        else :
            reader = xml_reader()
            doc = reader.fromUri(normalize_url(doc_uri)[0])
        try:
            ##xpath_lock.acquire()
            ##result = Evaluate(xpath,doc)
            ##xpath_lock.release()
            result = select_many(doc, xpath)
        except Exception,e:
            reader.releaseNode(doc)
            xmlrep_lock.release(doc_uri)
            raise e
    finally:
        xmlrep_lock.release(doc_uri)
        return doc,result,reader # we need all this to free memory afterwards
    

def write_tree_and_release(url,rw_module,doc,reader):
    xmlrep_lock.acquire(url)
    try:
        url,url_tuple = normalize_url(url)
        if url_tuple[0] == 'file' :
            if url_tuple[2][0]=='/' and url_tuple[2][2] == ':':
                fname = url_tuple[2][1:]
            else:
                fname = url_tuple[2]
        else:
            fname = url
        try:
            if rw_module :
                rw_module.write(doc,fname)
            else :
                f = open(url_tuple[2],'w')
                #from xml.dom.ext import PrettyPrint
                #PrettyPrint(doc,f)
                f.close
        except:
            import traceback
            traceback.print_exc()
            xmlrep_lock.release(url)
            raise Exception("Can't write file at URL %s" % url)
    finally:
        xmlrep_lock.release(url)
        reader.releaseNode(doc)

if __name__ == "__main__" :
    from xml.dom.ext.reader import Sax2
    doc=Sax2.FromXml(MOD_XML)
    print doc.documentElement
